@@ -7,12 +7,27 @@ use Sane 0.05;             # To get SANE_NAME_PAGE_WIDTH & SANE_NAME_PAGE_HEIGHT
 use Gscan2pdf::Dialog;
 use feature "switch";
 use Data::Dumper;
+my (
+ $_MAX_PAGES,        $_MAX_INCREMENT, $_DOUBLE_INCREMENT,
+ $_CANVAS_SIZE,      $_CANVAS_BORDER, $_CANVAS_POINT_SIZE,
+ $_CANVAS_MIN_WIDTH, $_NO_INDEX,      $EMPTY
+);
 
 # need to register this with Glib before we can use it below
 BEGIN {
  use Gscan2pdf::Scanner::Options;
  Glib::Type->register_enum( 'Gscan2pdf::Scanner::Dialog::Side',
   qw(facing reverse) );
+ use Readonly;
+ Readonly $_MAX_PAGES         => 999;
+ Readonly $_MAX_INCREMENT     => 99;
+ Readonly $_DOUBLE_INCREMENT  => 2;
+ Readonly $_CANVAS_SIZE       => 200;
+ Readonly $_CANVAS_BORDER     => 10;
+ Readonly $_CANVAS_POINT_SIZE => 10;
+ Readonly $_CANVAS_MIN_WIDTH  => 1;
+ Readonly $_NO_INDEX          => -1;
+ $EMPTY = q{};
 }
 
 # from http://gtk2-perl.sourceforge.net/doc/subclassing_widgets_in_perl.html
@@ -84,7 +99,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
   'device',                  # name
   'Device',                  # nick
   'Device name',             # blurb
-  '',                        # default
+  $EMPTY,                    # default
   [qw/readable writable/]    # flags
  ),
  Glib::ParamSpec->scalar(
@@ -115,7 +130,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
   'paper',                                      # name
   'Paper',                                      # nick
   'Name of currently selected paper format',    # blurb
-  '',                                           # default
+  $EMPTY,                                       # default
   [qw/readable writable/]                       # flags
  ),
  Glib::ParamSpec->scalar(
@@ -129,7 +144,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
   'Number of pages',                  # nickname
   'Number of pages to be scanned',    # blurb
   0,                                  # min 0 implies all
-  999,                                # max
+  $_MAX_PAGES,                        # max
   1,                                  # default
   [qw/readable writable/]             # flags
  ),
@@ -139,7 +154,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
 'Maximum number of pages that can be scanned with current page-number-start and page-number-increment'
   ,                                   # blurb
   -1,                                 # min -1 implies all
-  999,                                # max
+  $_MAX_PAGES,                        # max
   0,                                  # default
   [qw/readable writable/]             # flags
  ),
@@ -148,7 +163,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
   'Starting page number',                       # nickname
   'Page number of first page to be scanned',    # blurb
   1,                                            # min
-  999,                                          # max
+  $_MAX_PAGES,                                  # max
   1,                                            # default
   [qw/readable writable/]                       # flags
  ),
@@ -156,8 +171,8 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
   'page-number-increment',                                           # name
   'Page number increment',                                           # nickname
   'Amount to increment page number when scanning multiple pages',    # blurb
-  -99,                                                               # min
-  99,                                                                # max
+  -$_MAX_INCREMENT,                                                  # min
+  $_MAX_INCREMENT,                                                   # max
   1,                                                                 # default
   [qw/readable writable/]                                            # flags
  ),
@@ -187,9 +202,20 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::, signals => {
   'Visible scan options',                                  # nick
   'Hash of scan options to show or hide from the user',    # blurb
   [qw/readable writable/]                                  # flags
+ ),
+ Glib::ParamSpec->float(
+  'progress-pulse-step',                                   # name
+  'Progress pulse step',                                   # nick
+  'Pulse step of progress bar',                            # blurb
+  0.0,                                                     # minimum
+  1.0,                                                     # maximum
+  0.1,                                                     # default_value
+  [qw/readable writable/]                                  # flags
  ),
   ];
 
+our $VERSION = '1.2.3';
+
 my ( $d, $d_sane, $logger, $tooltips );
 my $tolerance = 1;
 
@@ -214,13 +240,13 @@ sub INIT_INSTANCE {
   changed => sub {
    my $index       = $self->{combobd}->get_active;
    my $device_list = $self->get('device-list');
-   if ( $index > $#$device_list ) {
+   if ( $index > $#{$device_list} ) {
     $self->{combobd}->hide;
     $labeld->hide;
     $self->set( 'device', undef );    # to make sure that the device is reloaded
     $self->get_devices;
    }
-   elsif ( $index > -1 ) {
+   elsif ( $index > $_NO_INDEX ) {
     $self->set( 'device', $device_list->[$index]{name} );
    }
   }
@@ -229,7 +255,7 @@ sub INIT_INSTANCE {
   'changed-device' => sub {
    my ( $widget, $device ) = @_;
    my $device_list = $self->get('device-list');
-   for (@$device_list) {
+   for ( @{$device_list} ) {
     if ( $_->{name} eq $device ) {
      Gscan2pdf::Dialog::Scan::set_combobox_by_text( $self->{combobd},
       $_->{label} );
@@ -269,25 +295,26 @@ sub INIT_INSTANCE {
  $vboxn->pack_start( $bscanall, TRUE, TRUE, 0 );
  $bscanall->signal_connect(
   clicked => sub {
-   $self->set( 'num-pages', 0 ) if ( $bscanall->get_active );
+   if ( $bscanall->get_active ) { $self->set( 'num-pages', 0 ) }
   }
  );
 
  # Entry button
  my $hboxn = Gtk2::HBox->new;
  $vboxn->pack_start( $hboxn, TRUE, TRUE, 0 );
- my $bscannum = Gtk2::RadioButton->new( $bscanall->get_group, "#:" );
+ my $bscannum = Gtk2::RadioButton->new( $bscanall->get_group, q{#:} );
  $tooltips->set_tip( $bscannum, $d->get('Set number of pages to scan') );
  $hboxn->pack_start( $bscannum, FALSE, FALSE, 0 );
 
  # Number of pages
- my $spin_buttonn = Gtk2::SpinButton->new_with_range( 1, 999, 1 );
+ my $spin_buttonn = Gtk2::SpinButton->new_with_range( 1, $_MAX_PAGES, 1 );
  $tooltips->set_tip( $spin_buttonn, $d->get('Set number of pages to scan') );
  $hboxn->pack_end( $spin_buttonn, FALSE, FALSE, 0 );
  $bscannum->signal_connect(
   clicked => sub {
-   $self->set( 'num-pages', $spin_buttonn->get_value )
-     if ( $bscannum->get_active );
+   if ( $bscannum->get_active ) {
+    $self->set( 'num-pages', $spin_buttonn->get_value );
+   }
   }
  );
  $self->signal_connect(
@@ -326,7 +353,7 @@ sub INIT_INSTANCE {
  $vboxx->pack_start( $hboxxs, FALSE, FALSE, 0 );
  my $labelxs = Gtk2::Label->new( $d->get('Start') );
  $hboxxs->pack_start( $labelxs, FALSE, FALSE, 0 );
- my $spin_buttons = Gtk2::SpinButton->new_with_range( 1, 99999, 1 );
+ my $spin_buttons = Gtk2::SpinButton->new_with_range( 1, $_MAX_PAGES, 1 );
  $hboxxs->pack_end( $spin_buttons, FALSE, FALSE, 0 );
  $spin_buttons->signal_connect(
   'value-changed' => sub {
@@ -345,13 +372,14 @@ sub INIT_INSTANCE {
  $vboxx->pack_start( $hboxi, FALSE, FALSE, 0 );
  my $labelxi = Gtk2::Label->new( $d->get('Increment') );
  $hboxi->pack_start( $labelxi, FALSE, FALSE, 0 );
- my $spin_buttoni = Gtk2::SpinButton->new_with_range( -99, 99, 1 );
+ my $spin_buttoni =
+   Gtk2::SpinButton->new_with_range( -$_MAX_INCREMENT, $_MAX_INCREMENT, 1 );
  $spin_buttoni->set_value( $self->get('page-number-increment') );
  $hboxi->pack_end( $spin_buttoni, FALSE, FALSE, 0 );
  $spin_buttoni->signal_connect(
   'value-changed' => sub {
    my $value = $spin_buttoni->get_value;
-   $value = -$self->get('page-number-increment') if ( $value == 0 );
+   if ( $value == 0 ) { $value = -$self->get('page-number-increment') }
    $spin_buttoni->set_value($value);
    $self->set( 'page-number-increment', $value );
   }
@@ -417,7 +445,8 @@ sub INIT_INSTANCE {
  $self->signal_connect(
   'changed-side-to-scan' => sub {
    my ( $widget, $value ) = @_;
-   $self->set( 'page-number-increment', $value eq 'facing' ? 2 : -2 );
+   $self->set( 'page-number-increment',
+    $value eq 'facing' ? $_DOUBLE_INCREMENT : -$_DOUBLE_INCREMENT );
   }
  );
  $tooltips->set_tip( $combobs,
@@ -431,7 +460,8 @@ sub INIT_INSTANCE {
  # Have to put the double-sided callback here to reference page side
  $buttond->signal_connect(
   clicked => sub {
-   $spin_buttoni->set_value( $combobs->get_active == 0 ? 2 : -2 );
+   $spin_buttoni->set_value(
+    $combobs->get_active == 0 ? $_DOUBLE_INCREMENT : -$_DOUBLE_INCREMENT );
   }
  );
 
@@ -451,7 +481,7 @@ sub INIT_INSTANCE {
  $self->{combobsp}->signal_connect(
   changed => sub {
    my $profile = $self->{combobsp}->get_active_text;
-   $self->set( 'profile', $profile ) if ( defined $profile );
+   if ( defined $profile ) { $self->set( 'profile', $profile ) }
   }
  );
  $vboxsp->pack_start( $self->{combobsp}, FALSE, FALSE, 0 );
@@ -478,11 +508,11 @@ sub INIT_INSTANCE {
    $dialog->set_default_response('ok');
    $dialog->show_all;
 
-   if ( $dialog->run eq 'ok' and $entry->get_text !~ /^\s*$/x ) {
+   if ( $dialog->run eq 'ok' and $entry->get_text !~ /^\s*$/xsm ) {
     my $profile = $entry->get_text;
     $self->add_profile( $profile, $self->get('current-scan-options') );
-    $self->{combobsp}->set_active(
-     Gscan2pdf::Dialog::Scan::num_rows_combobox( $self->{combobsp} ) );
+    $self->{combobsp}
+      ->set_active( get_combobox_num_rows( $self->{combobsp} ) - 1 );
    }
    $dialog->destroy;
   }
@@ -520,7 +550,7 @@ sub INIT_INSTANCE {
 
  $self->signal_connect(
   check_resize => sub {
-   Glib::Idle->add( sub { $self->resize( 100, 100 ); } );
+   Glib::Idle->add( sub { $self->resize( 1, 1 ); } );
   }
  );
  return $self;
@@ -531,21 +561,28 @@ sub SET_PROPERTY {
  my $name   = $pspec->get_name;
  my $oldval = $self->get($name);
  $self->{$name} = $newval;
- if (( defined($newval) and defined($oldval) and $newval ne $oldval )
+
+ # Have to set logger separately as it has already been set in the subclassed
+ # widget
+ if ( $name eq 'logger' ) {
+  $logger = $newval;
+  $logger->debug('Set logger in Gscan2pdf::Dialog::Scan');
+ }
+ elsif ( ( defined($newval) and defined($oldval) and $newval ne $oldval )
   or ( defined($newval) xor defined($oldval) ) )
  {
   if ( defined $logger ) {
    $logger->debug(
-    "Setting $name from "
+    "Started setting $name from "
       . (
      defined($oldval)
-     ? ( ref($oldval) =~ /(?:HASH|ARRAY)/x ? Dumper($oldval) : $oldval )
+     ? ( ref($oldval) =~ /(?:HASH|ARRAY)/xsm ? Dumper($oldval) : $oldval )
      : 'undef'
       )
       . ' to '
       . (
      defined($newval)
-     ? ( ref($newval) =~ /(?:HASH|ARRAY)/x ? Dumper($newval) : $newval )
+     ? ( ref($newval) =~ /(?:HASH|ARRAY)/xsm ? Dumper($newval) : $newval )
      : 'undef'
       )
    );
@@ -559,7 +596,6 @@ sub SET_PROPERTY {
     $self->set_device_list($newval);
     $self->signal_emit( 'changed-device-list', $newval )
    }
-   when ('logger') { $logger = $self->get('logger') }
    when ('num_pages') { $self->signal_emit( 'changed-num-pages', $newval ) }
    when ('page_number_start') {
     $self->signal_emit( 'changed-page-number-start', $newval )
@@ -600,6 +636,22 @@ sub SET_PROPERTY {
     $self->SUPER::SET_PROPERTY( $pspec, $newval );
    }
   }
+  if ( defined $logger ) {
+   $logger->debug(
+    "Finished setting $name from "
+      . (
+     defined($oldval)
+     ? ( ref($oldval) =~ /(?:HASH|ARRAY)/xsm ? Dumper($oldval) : $oldval )
+     : 'undef'
+      )
+      . ' to '
+      . (
+     defined($newval)
+     ? ( ref($newval) =~ /(?:HASH|ARRAY)/xsm ? Dumper($newval) : $newval )
+     : 'undef'
+      )
+   );
+  }
  }
  return;
 }
@@ -608,25 +660,20 @@ sub show {
  my $self = shift;
  $self->signal_chain_from_overridden;
  $self->{framex}->hide_all;
+ if ( $self->{combobp}->get_active_text ne $d->get('Manual') ) {
+  $self->hide_geometry( $self->get('available-scan-options') );
+ }
  return;
 }
 
-# Get number of rows in combobox
-sub num_rows_combobox {
- my ($combobox) = @_;
- my $i = -1;
- $combobox->get_model->foreach( sub { $i++; return FALSE } );
- return $i;
-}
-
 sub set_device {
  my ( $self, $device ) = @_;
- if ( defined($device) and $device ne '' ) {
+ if ( defined($device) and $device ne $EMPTY ) {
   my $o;
   my $device_list = $self->get('device_list');
   if ( defined $device_list ) {
-   for ( my $i = 0 ; $i < @$device_list ; $i++ ) {
-    $o = $i if ( $device eq $device_list->[$i]{name} );
+   for ( 0 .. $#{$device_list} ) {
+    if ( $device eq $device_list->[$_]{name} ) { $o = $_ }
    }
 
    # Set the device dependent options after the number of pages
@@ -652,10 +699,10 @@ sub set_device_list {
  # Note any duplicate device names and delete if necessary
  my %seen;
  my $i = 0;
- while ( $i < @$device_list ) {
+ while ( $i < @{$device_list} ) {
   $seen{ $device_list->[$i]{name} }++;
   if ( $seen{ $device_list->[$i]{name} } > 1 ) {
-   splice @$device_list, $i, 1;
+   splice @{$device_list}, $i, 1;
   }
   else {
    $i++;
@@ -664,36 +711,63 @@ sub set_device_list {
 
  # Note any duplicate model names and add the device if necessary
  undef %seen;
- for (@$device_list) {
-  $_->{model} = $_->{name} unless ( defined $_->{model} );
+ for ( @{$device_list} ) {
+  if ( not defined( $_->{model} ) ) { $_->{model} = $_->{name} }
   $seen{ $_->{model} }++;
  }
- for (@$device_list) {
+ for ( @{$device_list} ) {
   if ( defined $_->{vendor} ) {
    $_->{label} = "$_->{vendor} $_->{model}";
   }
   else {
    $_->{label} = $_->{model};
   }
-  $_->{label} .= " on $_->{name}" if ( $seen{ $_->{model} } > 1 );
+  if ( $seen{ $_->{model} } > 1 ) { $_->{label} .= " on $_->{name}" }
  }
 
  $self->{combobd}->signal_handler_block( $self->{combobd_changed_signal} );
 
  # Remove all entries apart from rescan
- for ( my $j = get_combobox_num_items( $self->{combobd} ) ; $j > 1 ; $j-- ) {
+ for ( get_combobox_num_rows( $self->{combobd} ) .. 2 ) {
   $self->{combobd}->remove_text(0);
  }
 
  # read the model names into the combobox
- for ( my $j = 0 ; $j < @$device_list ; $j++ ) {
-  $self->{combobd}->insert_text( $j, $device_list->[$j]{label} );
+ for ( 0 .. $#{$device_list} ) {
+  $self->{combobd}->insert_text( $_, $device_list->[$_]{label} );
  }
 
  $self->{combobd}->signal_handler_unblock( $self->{combobd_changed_signal} );
  return;
 }
 
+sub pack_widget {
+ my ( $self, $widget, $data ) = @_;
+ my ( $options, $opt, $hbox, $hboxp ) = @{$data};
+ if ( defined $widget ) {
+  $opt->{widget} = $widget;
+  if ( $opt->{type} == SANE_TYPE_BUTTON or $opt->{max_values} > 1 ) {
+   $hbox->pack_end( $widget, TRUE, TRUE, 0 );
+  }
+  else {
+   $hbox->pack_end( $widget, FALSE, FALSE, 0 );
+  }
+  $tooltips->set_tip( $widget, $d_sane->get( $opt->{desc} ) );
+
+  # Look-up to hide/show the box if necessary
+  if ( $self->_geometry_option($opt) ) {
+   $options->{box}{ $opt->{name} } = $hbox;
+  }
+
+  $self->create_paper_widget( $options, $hboxp );
+
+ }
+ else {
+  $logger->warn("Unknown type $opt->{type}");
+ }
+ return;
+}
+
 # Add paper size to combobox if scanner large enough
 
 sub set_paper_formats {
@@ -701,7 +775,7 @@ sub set_paper_formats {
  $self->{ignored_paper_formats} = ();
  my $options = $self->get('available-scan-options');
 
- for ( keys %$formats ) {
+ for ( keys %{$formats} ) {
   if ( defined( $self->{combobp} )
    and $options->supports_paper( $formats->{$_}, $tolerance ) )
   {
@@ -747,7 +821,7 @@ sub edit_paper {
   $d->get('Left')   => 'int',
   $d->get('Top')    => 'int'
  );
- for ( keys %$formats ) {
+ for ( keys %{$formats} ) {
   push @{ $slist->{data} },
     [
    $_,                $formats->{$_}{x}, $formats->{$_}{y},
@@ -756,7 +830,8 @@ sub edit_paper {
  }
 
  # Set everything to be editable
- for ( 0 .. 4 ) {
+ my @columns = $slist->get_columns;
+ for ( 0 .. $#columns ) {
   $slist->set_column_editable( $_, TRUE );
  }
  $slist->get_column(0)->set_sort_column_id(0);
@@ -765,7 +840,7 @@ sub edit_paper {
  $dbutton->signal_connect(
   clicked => sub {
    my @rows = $slist->get_selected_indices;
-   $rows[0] = 0 if ( !@rows );
+   if ( not @rows ) { $rows[0] = 0 }
    my $name    = $slist->{data}[ $rows[0] ][0];
    my $version = 2;
    my $i       = 0;
@@ -778,14 +853,11 @@ sub edit_paper {
      ++$i;
     }
    }
-   my @line = [
-    "$name ($version)",
-    $slist->{data}[ $rows[0] ][1],
-    $slist->{data}[ $rows[0] ][2],
-    $slist->{data}[ $rows[0] ][3],
-    $slist->{data}[ $rows[0] ][4]
-   ];
-   splice @{ $slist->{data} }, $rows[0] + 1, 0, @line;
+   my @line = ("$name ($version)");
+   for ( 1 .. $#{ $slist->get_columns } ) {
+    push @line, $slist->{data}[ $rows[0] ][$_];
+   }
+   splice @{ $slist->{data} }, $rows[0] + 1, 0, \@line;
   }
  );
 
@@ -809,9 +881,9 @@ sub edit_paper {
  $slist->get_model->signal_connect(
   'row-changed' => sub {
    my ( $model, $path, $iter ) = @_;
-   for ( my $i = 0 ; $i < @{ $slist->{data} } ; $i++ ) {
-    if ( $i != $path->to_string
-     and $slist->{data}[ $path->to_string ][0] eq $slist->{data}[$i][0] )
+   for ( 0 .. $#{ $slist->{data} } ) {
+    if ( $_ != $path->to_string
+     and $slist->{data}[ $path->to_string ][0] eq $slist->{data}[$_][0] )
     {
      my $name    = $slist->{data}[ $path->to_string ][0];
      my $version = 2;
@@ -821,7 +893,7 @@ sub edit_paper {
                      \ \( # space, opening bracket
                      (\d+) # version
                      \) # closing bracket
-                   /x
+                   /xsm
        )
      {
       $name    = $1;
@@ -842,30 +914,32 @@ sub edit_paper {
  $abutton->signal_connect(
   clicked => sub {
    my %formats;
-   for ( my $i = 0 ; $i < @{ $slist->{data} } ; $i++ ) {
-    $formats{ $slist->{data}[$i][0] }{x} = $slist->{data}[$i][1];
-    $formats{ $slist->{data}[$i][0] }{y} = $slist->{data}[$i][2];
-    $formats{ $slist->{data}[$i][0] }{l} = $slist->{data}[$i][3];
-    $formats{ $slist->{data}[$i][0] }{t} = $slist->{data}[$i][4];
+   for my $i ( 0 .. $#{ $slist->{data} } ) {
+    my $j = 0;
+    for (qw( x y l t)) {
+     $formats{ $slist->{data}[$i][0] }{$_} = $slist->{data}[$i][ ++$j ];
+    }
    }
 
    # Remove all formats, leaving Manual and Edit
-   $combobp->remove_text(0) while ( $combobp->get_active > 1 );
+   while ( $combobp->get_active > 1 ) { $combobp->remove_text(0) }
 
    # Add new definitions
    $self->set( 'paper-formats', \%formats );
-   main::show_message_dialog(
-    $window,
-    'warning',
-    'close',
-    $d->get(
+   if ( $self->{ignored_paper_formats}
+    and @{ $self->{ignored_paper_formats} } )
+   {
+    main::show_message_dialog(
+     $window,
+     'warning',
+     'close',
+     $d->get(
 'The following paper sizes are too big to be scanned by the selected device:'
-      )
-      . ' '
-      . join( ', ', @{ $self->{ignored_paper_formats} } )
-     )
-     if ( $self->{ignored_paper_formats}
-    and @{ $self->{ignored_paper_formats} } );
+       )
+       . q{ }
+       . join( ', ', @{ $self->{ignored_paper_formats} } )
+    );
+   }
 
    # Set the combobox back from Edit to the previous value
    set_combobox_by_text( $combobp, $self->get('paper') );
@@ -894,12 +968,12 @@ sub add_profile {
  if ( defined($name) and defined($profile) ) {
   $self->{profiles}{$name} = ();
   if ( ref($profile) eq 'ARRAY' ) {
-   for (@$profile) {
+   for ( @{$profile} ) {
     push @{ $self->{profiles}{$name} }, $_;
    }
   }
   elsif ( ref($profile) eq 'HASH' ) {
-   while ( my ( $key, $value ) = each(%$profile) ) {
+   while ( my ( $key, $value ) = each( %{$profile} ) ) {
     push @{ $self->{profiles}{$name} }, { $key => $value };
    }
   }
@@ -912,7 +986,7 @@ sub add_profile {
 sub set_profile {
  my ( $self, $name ) = @_;
  set_combobox_by_text( $self->{combobsp}, $name );
- if ( defined($name) and $name ne '' ) {
+ if ( defined($name) and $name ne $EMPTY ) {
 
   # If we are setting the profile, don't unset the profile name
   $self->{setting_profile} = TRUE;
@@ -943,8 +1017,10 @@ sub remove_profile {
  my ( $self, $name ) = @_;
  if ( defined($name) and defined( $self->{profiles}{$name} ) ) {
   my $i = get_combobox_by_text( $self->{combobsp}, $name );
-  if ( $i > -1 ) {
-   $self->{combobsp}->set_active(-1) if ( $self->{combobsp}->get_active == $i );
+  if ( $i > $_NO_INDEX ) {
+   if ( $self->{combobsp}->get_active == $i ) {
+    $self->{combobsp}->set_active($_NO_INDEX);
+   }
    $self->{combobsp}->remove_text($i);
    $self->signal_emit( 'removed-profile', $name );
   }
@@ -952,9 +1028,9 @@ sub remove_profile {
  return;
 }
 
-sub get_combobox_num_items {
+sub get_combobox_num_rows {
  my ($combobox) = @_;
- return unless ( defined $combobox );
+ if ( not defined($combobox) ) { return }
  my $i = 0;
  $combobox->get_model->foreach(
   sub {
@@ -967,8 +1043,8 @@ sub get_combobox_num_items {
 
 sub get_combobox_by_text {
  my ( $combobox, $text ) = @_;
- return -1 unless ( defined($combobox) and defined($text) );
- my $o = -1;
+ if ( not( defined($combobox) and defined($text) ) ) { return $_NO_INDEX }
+ my $o = $_NO_INDEX;
  my $i = 0;
  $combobox->get_model->foreach(
   sub {
@@ -988,14 +1064,16 @@ sub get_combobox_by_text {
 
 sub set_combobox_by_text {
  my ( $combobox, $text ) = @_;
- $combobox->set_active( get_combobox_by_text( $combobox, $text ) )
-   if ( defined $combobox );
+ if ( defined $combobox ) {
+  $combobox->set_active( get_combobox_by_text( $combobox, $text ) );
+ }
  return;
 }
 
 sub _extended_pagenumber_checkbox_callback {
  my ( $widget, $data ) = @_;
- my ( $dialog, $frames, $spin_buttoni, $buttons, $buttond, $combobs ) = @$data;
+ my ( $dialog, $frames, $spin_buttoni, $buttons, $buttond, $combobs ) =
+   @{$data};
  if ( $widget->get_active ) {
   $frames->hide_all;
   $dialog->{framex}->show_all;
@@ -1020,7 +1098,7 @@ sub _extended_pagenumber_checkbox_callback {
 
 sub multiple_values_button_callback {
  my ( $widget, $data ) = @_;
- my ( $dialog, $opt )  = @$data;
+ my ( $dialog, $opt )  = @{$data};
  if ($opt->{type} == SANE_TYPE_FIXED
   or $opt->{type} == SANE_TYPE_INT )
  {
@@ -1066,9 +1144,8 @@ sub set_options {
  );
  my $vbox   = $window->vbox;
  my $canvas = Goo::Canvas->new;
- my ( $cwidth, $cheight ) = ( 200, 200 );
- $canvas->set_size_request( $cwidth, $cheight );
- $canvas->{border} = 10;
+ $canvas->set_size_request( $_CANVAS_SIZE, $_CANVAS_SIZE );
+ $canvas->{border} = $_CANVAS_BORDER;
  $vbox->add($canvas);
  my $root = $canvas->get_root_item;
 
@@ -1161,7 +1238,7 @@ sub set_options {
 sub add_value {
  my ( $root, $canvas ) = @_;
  my $item = Goo::Canvas::Rect->new(
-  $root, 0, 0, 10, 10,
+  $root, 0, 0, $_CANVAS_POINT_SIZE, $_CANVAS_POINT_SIZE,
   'fill-color' => 'black',
   'line-width' => 0,
  );
@@ -1200,12 +1277,16 @@ sub add_value {
  $item->signal_connect(
   'motion-notify-event' => sub {
    my ( $widget, $target, $event ) = @_;
-   return FALSE
-     unless ## no critic (ProhibitNegativeExpressionsInUnlessAndUntilConditions)
-     (
-    $event->state >=    ## no critic (ProhibitMismatchedOperators)
-    'button1-mask'
-     );
+   if (
+    not(
+     $event->state >=    ## no critic (ProhibitMismatchedOperators)
+     'button1-mask'
+    )
+     )
+   {
+    return FALSE;
+   }
+
    my ( $x, $y ) = ( $event->x, $event->y );
    my ( $xgr, $ygr ) = ( 0, $y );
    if ( $opt->{constraint_type} == SANE_CONSTRAINT_RANGE ) {
@@ -1219,20 +1300,20 @@ sub add_value {
    }
    elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_WORD_LIST ) {
     ( $xgr, $ygr ) = to_graph( $canvas, 0, $y );
-    for ( my $i = 1 ; $i < @{ $opt->{constraint} } ; $i++ ) {
-     if ( $ygr < ( $opt->{constraint}[$i] + $opt->{constraint}[ $i - 1 ] ) / 2 )
+    for ( 1 .. $#{ $opt->{constraint} } ) {
+     if ( $ygr < ( $opt->{constraint}[$_] + $opt->{constraint}[ $_ - 1 ] ) / 2 )
      {
-      $ygr = $opt->{constraint}[ $i - 1 ];
+      $ygr = $opt->{constraint}[ $_ - 1 ];
       last;
      }
-     elsif ( $i == $#{ $opt->{constraint} } ) {
-      $ygr = $opt->{constraint}[$i];
+     elsif ( $_ == $#{ $opt->{constraint} } ) {
+      $ygr = $opt->{constraint}[$_];
      }
     }
    }
    $canvas->{val}[ $widget->{index} ] = $ygr;
    ( $x, $y ) = to_canvas( $canvas, $xgr, $ygr );
-   $widget->set( y => $y - 10 / 2 );
+   $widget->set( y => $y - $_CANVAS_POINT_SIZE / 2 );
    return TRUE;
   }
  );
@@ -1262,28 +1343,28 @@ sub update_graph {
  my ($canvas) = @_;
 
  # Calculate bounds of graph
- my @bounds;
+ my ( @xbounds, @ybounds );
  for ( @{ $canvas->{val} } ) {
-  $bounds[1] = $_ if ( not defined $bounds[1] or $_ < $bounds[1] );
-  $bounds[3] = $_ if ( not defined $bounds[3] or $_ > $bounds[3] );
+  if ( not defined $ybounds[0] or $_ < $ybounds[0] ) { $ybounds[0] = $_ }
+  if ( not defined $ybounds[1] or $_ > $ybounds[1] ) { $ybounds[1] = $_ }
  }
  my $opt = $canvas->{opt};
- $bounds[0] = 0;
- $bounds[2] = $#{ $canvas->{val} };
- if ( $bounds[0] >= $bounds[2] ) {
-  $bounds[0] = -0.5;
-  $bounds[2] = 0.5;
+ $xbounds[0] = 0;
+ $xbounds[1] = $#{ $canvas->{val} };
+ if ( $xbounds[0] >= $xbounds[1] ) {
+  $xbounds[0] = -$_CANVAS_MIN_WIDTH / 2;
+  $xbounds[1] = $_CANVAS_MIN_WIDTH / 2;
  }
  if ( $opt->{constraint_type} == SANE_CONSTRAINT_RANGE ) {
-  $bounds[1] = $opt->{constraint}{min};
-  $bounds[3] = $opt->{constraint}{max};
+  $ybounds[0] = $opt->{constraint}{min};
+  $ybounds[1] = $opt->{constraint}{max};
  }
  elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_WORD_LIST ) {
-  $bounds[1] = $opt->{constraint}[0];
-  $bounds[3] = $opt->{constraint}[ $#{ $opt->{constraint} } ];
+  $ybounds[0] = $opt->{constraint}[0];
+  $ybounds[1] = $opt->{constraint}[ $#{ $opt->{constraint} } ];
  }
  my ( $vwidth, $vheight ) =
-   ( $bounds[2] - $bounds[0], $bounds[3] - $bounds[1] );
+   ( $xbounds[1] - $xbounds[0], $ybounds[1] - $ybounds[0] );
 
  # Calculate bounds of canvas
  my ( $x, $y, $cwidth, $cheight ) = $canvas->allocation->values;
@@ -1295,15 +1376,15 @@ sub update_graph {
  );
 
  $canvas->{scale}   = \@scale;
- $canvas->{bounds}  = \@bounds;
+ $canvas->{bounds}  = [ $xbounds[0], $ybounds[0], $xbounds[1], $xbounds[1] ];
  $canvas->{cheight} = $cheight;
 
  # Update canvas
- for ( my $i = 0 ; $i <= $#{ $canvas->{items} } ; $i++ ) {
-  my $item = $canvas->{items}[$i];
-  $item->{index} = $i;
-  my ( $xc, $yc ) = to_canvas( $canvas, $i, $canvas->{val}[$i] );
-  $item->set( x => $xc - 10 / 2, y => $yc - 10 / 2 );
+ for ( 0 .. $#{ $canvas->{items} } ) {
+  my $item = $canvas->{items}[$_];
+  $item->{index} = $_;
+  my ( $xc, $yc ) = to_canvas( $canvas, $_, $canvas->{val}[$_] );
+  $item->set( x => $xc - $_CANVAS_BORDER / 2, y => $yc - $_CANVAS_BORDER / 2 );
  }
  return;
 }
@@ -1314,12 +1395,12 @@ sub my_dumper {
  my ($ref) = @_;
  given ( ref $ref ) {
   when ('ARRAY') {
-   for (@$ref) {
+   for ( @{$ref} ) {
     my_dumper($_);
    }
   }
   when ('HASH') {
-   while ( my ( $key, $val ) = each(%$ref) ) {
+   while ( my ( $key, $val ) = each( %{$ref} ) ) {
     my_dumper($val);
    }
   }
@@ -1331,7 +1412,7 @@ sub my_dumper {
 
 sub set_option_emit_signal {
  my ( $self, $i, $defaults, $signal1, $signal2 ) = @_;
- $i = $self->set_option_widget( $i, $defaults ) if ( $i < @$defaults );
+ if ( $i < @{$defaults} ) { $i = $self->set_option_widget( $i, $defaults ) }
 
  # Only emit the changed-current-scan-options signal when we have finished
  if ( ( not defined($i) or $i > $#{$defaults} )
@@ -1340,7 +1421,7 @@ sub set_option_emit_signal {
  {
   $self->signal_handler_disconnect($signal1);
   $self->signal_handler_disconnect($signal2);
-  $self->set( 'profile', undef ) unless ( $self->{setting_profile} );
+  if ( not $self->{setting_profile} ) { $self->set( 'profile', undef ) }
   $self->signal_emit( 'changed-current-scan-options',
    $self->get('current-scan-options') );
  }
@@ -1355,8 +1436,8 @@ sub get_option_from_profile {
  # for reasons I don't understand, without walking the reference tree,
  # parts of $profile are undef
  my_dumper($profile);
- for (@$profile) {
-  my ( $key, $val ) = each(%$_);
+ for ( @{$profile} ) {
+  my ( $key, $val ) = each( %{$_} );
   return $val if ( $key eq $name );
  }
  return;
