@@ -11,6 +11,10 @@ use Gscan2pdf::Frontend::CLI;
 use Storable qw(dclone);     # For cloning the options cache
 use Locale::gettext 1.05;    # For translations
 use feature "switch";
+use List::MoreUtils qw{any};
+
+my $EMPTY = q{};
+my $COMMA = q{,};
 
 # logger duplicated from Gscan2pdf::Dialog::Scan
 # to ensure that SET_PROPERTIES gets called in both places
@@ -30,7 +34,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, signals => {
   'frontend',                                           # name
   'Frontend',                                           # nick
   '(scanimage|scanadf)(-perl)?',                        # blurb
-  '',                                                   # default
+  'scanimage',                                          # default
   [qw/readable writable/]                               # flags
  ),
  Glib::ParamSpec->scalar(
@@ -43,7 +47,7 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, signals => {
   'prefix',                                             # name
   'Prefix',                                             # nick
   'Prefix for command line calls',                      # blurb
-  '',                                                   # default
+  $EMPTY,                                               # default
   [qw/readable writable/]                               # flags
  ),
  Glib::ParamSpec->scalar(
@@ -67,6 +71,8 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, signals => {
  ),
   ];
 
+our $VERSION = '1.2.3';
+
 my $SANE_NAME_PAGE_HEIGHT = SANE_NAME_PAGE_HEIGHT;
 my $SANE_NAME_PAGE_WIDTH  = SANE_NAME_PAGE_WIDTH;
 my ( $d, $d_sane, $logger, $tooltips );
@@ -89,7 +95,10 @@ sub SET_PROPERTY {
  if (( defined($newval) and defined($oldval) and $newval ne $oldval )
   or ( defined($newval) xor defined($oldval) ) )
  {
-  if ( $name eq 'logger' ) { $logger = $self->get('logger') }
+  if ( $name eq 'logger' ) {
+   $logger = $newval;
+   $logger->debug('Set logger in Gscan2pdf::Dialog::Scan::CLI');
+  }
   elsif ( $name eq 'cache_options' ) {
    $self->signal_emit( 'changed-cache-options', $newval );
   }
@@ -114,7 +123,7 @@ sub get_devices {
 
    # Set up ProgressBar
    $pbar = Gtk2::ProgressBar->new;
-   $pbar->set_pulse_step(.1);
+   $pbar->set_pulse_step( $self->get('progress-pulse-step') );
    $pbar->set_text( $d->get('Fetching list of devices') );
    $hboxd->pack_start( $pbar, TRUE, TRUE, 0 );
    $hboxd->hide_all;
@@ -147,47 +156,48 @@ sub get_devices {
 
 sub cache_key {
  my ( $self, $options ) = @_;
- my $cache_key = '';
 
  my $reload_triggers = $self->get('reload-triggers');
- if ( defined $reload_triggers ) {
-  $reload_triggers = [$reload_triggers]
-    if ( ref($reload_triggers) ne 'ARRAY' );
+ if ( not defined($reload_triggers) ) { return 'default' }
 
-  if ( defined $options ) {
-
-   for my $opt ( @{ $options->{array} } ) {
-    for (@$reload_triggers) {
-     if ( defined( $opt->{name} ) and /^$opt->{name}$/ix ) {
-      $cache_key .= ',' unless ( $cache_key eq '' );
-      $cache_key .= "$opt->{name},$opt->{val}";
-      last;
-     }
-    }
-   }
-  }
-  else {
-
-   # for reasons I don't understand, without walking the reference tree,
-   # parts of $default are undef
-   Gscan2pdf::Dialog::Scan::my_dumper( $self->{current_scan_options} );
-
-   # grep the reload triggers from the current options
-   for ( @{ $self->{current_scan_options} } ) {
-    my ( $key, $value ) = each(%$_);
-    for (@$reload_triggers) {
-     if (/^$key$/ix) {
-      $cache_key .= ',' unless ( $cache_key eq '' );
-      $cache_key .= "$key,$value";
-      last;
-     }
-    }
-   }
-
-  }
+ if ( ref($reload_triggers) ne 'ARRAY' ) {
+  $reload_triggers = [$reload_triggers];
  }
 
- $cache_key = 'default' if ( $cache_key eq '' );
+ my $cache_key = $EMPTY;
+ if ( defined $options ) {
+
+  for my $opt ( @{ $options->{array} } ) {
+   for ( @{$reload_triggers} ) {
+    if ( defined( $opt->{name} ) and /^$opt->{name}$/ixsm ) {
+     if ( $cache_key ne $EMPTY ) { $cache_key .= $COMMA }
+     $cache_key .= "$opt->{name},$opt->{val}";
+     last;
+    }
+   }
+  }
+ }
+ else {
+
+  # for reasons I don't understand, without walking the reference tree,
+  # parts of $default are undef
+  Gscan2pdf::Dialog::Scan::my_dumper( $self->{current_scan_options} );
+
+  # grep the reload triggers from the current options
+  for ( @{ $self->{current_scan_options} } ) {
+   my ( $key, $value ) = each( %{$_} );
+   for ( @{$reload_triggers} ) {
+    if (/^$key$/ixsm) {
+     if ( $cache_key ne $EMPTY ) { $cache_key .= $COMMA }
+     $cache_key .= "$key,$value";
+     last;
+    }
+   }
+  }
+
+ }
+
+ if ( $cache_key eq $EMPTY ) { $cache_key = 'default' }
  return $cache_key;
 }
 
@@ -198,11 +208,13 @@ sub scan_options {
 
  # Remove any existing pages
  while ( $self->{notebook}->get_n_pages > 1 ) {
-  $self->{notebook}->remove_page(-1);
+
+  # -1 = last page
+  $self->{notebook}->remove_page(-1);    ## no critic (ProhibitMagicNumbers)
  }
 
  # Ghost the scan button whilst options being updated
- $self->{sbutton}->set_sensitive(FALSE) if ( defined $self->{sbutton} );
+ if ( defined $self->{sbutton} ) { $self->{sbutton}->set_sensitive(FALSE) }
 
  my ( $pbar, $cache_key );
  my $hboxd = $self->{hboxd};
@@ -236,7 +248,7 @@ sub scan_options {
 
    # Set up ProgressBar
    $pbar = Gtk2::ProgressBar->new;
-   $pbar->set_pulse_step(.1);
+   $pbar->set_pulse_step( $self->get('progress-pulse-step') );
    $pbar->set_text( $d->get('Updating options') );
    $hboxd->pack_start( $pbar, TRUE, TRUE, 0 );
    $hboxd->hide_all;
@@ -256,7 +268,7 @@ sub scan_options {
 
     # Don't assume that the options we have are those we are looking for yet
     # Recalculating cache_key based on contents of options
-    $cache_key = $self->cache_key($options) unless ( $cache_key eq 'default' );
+    if ( $cache_key ne 'default' ) { $cache_key = $self->cache_key($options) }
 
     # We only store the array part of the options object
     # as we have to recreate the object anyway when we retrieve it
@@ -313,19 +325,18 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
  $self->{notebook}->append_page( $vbox, $d->get('Scan Options') );
 
  delete $self->{combobp};    # So we don't carry over from one device to another
- for ( my $i = 1 ; $i < $num_dev_options ; ++$i ) {
-  my $opt = $options->by_index($i);
+ for ( 1 .. $num_dev_options - 1 ) {
+  my $opt = $options->by_index($_);
 
   # Notebook page for group
   if ( $opt->{type} == SANE_TYPE_GROUP ) {
    $vbox = Gtk2::VBox->new;
-   my $i =
-     $self->{notebook}->append_page( $vbox, $d_sane->get( $opt->{title} ) );
+   $self->{notebook}->append_page( $vbox, $d_sane->get( $opt->{title} ) );
    $opt->{widget} = $vbox;
    next;
   }
 
-  next unless ( $opt->{cap} & SANE_CAP_SOFT_DETECT );
+  if ( not( $opt->{cap} & SANE_CAP_SOFT_DETECT ) ) { next }
 
   # Widget
   my ( $widget, $val );
@@ -333,7 +344,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
 
   # Define HBox for paper size here
   # so that it can be put before first geometry option
-  if ( _geometry_option($opt) and not defined( $self->{hboxp} ) ) {
+  if ( $self->_geometry_option($opt) and not defined( $self->{hboxp} ) ) {
    $self->{hboxp} = Gtk2::HBox->new;
    $vbox->pack_start( $self->{hboxp}, FALSE, FALSE, 0 );
   }
@@ -341,9 +352,11 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
   # HBox for option
   my $hbox = Gtk2::HBox->new;
   $vbox->pack_start( $hbox, FALSE, TRUE, 0 );
-  $hbox->set_sensitive(FALSE)
-    if ( $opt->{cap} & SANE_CAP_INACTIVE
-   or not $opt->{cap} & SANE_CAP_SOFT_SELECT );
+  if ( $opt->{cap} & SANE_CAP_INACTIVE
+   or not $opt->{cap} & SANE_CAP_SOFT_SELECT )
+  {
+   $hbox->set_sensitive(FALSE);
+  }
 
   if ( $opt->{max_values} < 2 ) {
 
@@ -357,7 +370,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
    if ( $opt->{type} == SANE_TYPE_BOOL )
    {    ## no critic (ProhibitCascadingIfElse)
     $widget = Gtk2::CheckButton->new;
-    $widget->set_active(TRUE) if ($val);
+    if ($val) { $widget->set_active(TRUE) }
     $widget->{signal} = $widget->signal_connect(
      toggled => sub {
       my $value = $widget->get_active;
@@ -379,13 +392,14 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
    # SpinButton
    elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_RANGE ) {
     my $step = 1;
-    $step = $opt->{constraint}{quant} if ( $opt->{constraint}{quant} );
+    if ( $opt->{constraint}{quant} ) { $step = $opt->{constraint}{quant} }
     $widget = Gtk2::SpinButton->new_with_range( $opt->{constraint}{min},
      $opt->{constraint}{max}, $step );
 
     # Set the default
-    $widget->set_value($val)
-      if ( defined $val and not $opt->{cap} & SANE_CAP_INACTIVE );
+    if ( defined $val and not $opt->{cap} & SANE_CAP_INACTIVE ) {
+     $widget->set_value($val);
+    }
     $widget->{signal} = $widget->signal_connect(
      'value-changed' => sub {
       my $value = $widget->get_value;
@@ -400,13 +414,13 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
    {
     $widget = Gtk2::ComboBox->new_text;
     my $index = 0;
-    for ( my $i = 0 ; $i < @{ $opt->{constraint} } ; ++$i ) {
-     $widget->append_text( $d_sane->get( $opt->{constraint}[$i] ) );
-     $index = $i if ( defined $val and $opt->{constraint}[$i] eq $val );
+    for ( 0 .. $#{ $opt->{constraint} } ) {
+     $widget->append_text( $d_sane->get( $opt->{constraint}[$_] ) );
+     if ( defined $val and $opt->{constraint}[$_] eq $val ) { $index = $_ }
     }
 
     # Set the default
-    $widget->set_active($index) if ( defined $index );
+    if ( defined $index ) { $widget->set_active($index) }
     $widget->{signal} = $widget->signal_connect(
      changed => sub {
       my $i = $widget->get_active;
@@ -420,8 +434,9 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
     $widget = Gtk2::Entry->new;
 
     # Set the default
-    $widget->set_text($val)
-      if ( defined $val and not $opt->{cap} & SANE_CAP_INACTIVE );
+    if ( defined $val and not $opt->{cap} & SANE_CAP_INACTIVE ) {
+     $widget->set_text($val);
+    }
     $widget->{signal} = $widget->signal_connect(
      activate => sub {
       my $value = $widget->get_text;
@@ -438,7 +453,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
    );
   }
 
-  $self->_pack_widget( $widget, [ $options, $opt, $hbox ] );
+  $self->pack_widget( $widget, [ $options, $opt, $hbox ] );
  }
 
  # Set defaults
@@ -454,9 +469,6 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
  $self->_update_option_visibility( $options,
   $self->get('visible-scan-options') );
 
- # Give the GUI a chance to catch up before resizing.
- Glib::Idle->add( sub { $self->resize( 100, 100 ); } );
-
  $self->{sbutton}->set_sensitive(TRUE);
  $self->{sbutton}->grab_focus;
  return;
@@ -466,13 +478,13 @@ sub _update_option_visibility {
  my ( $self, $options, $visible_options ) = @_;
 
  # Show all notebook tabs
- for ( my $i = 1 ; $i < $self->{notebook}->get_n_pages ; $i++ ) {
-  $self->{notebook}->get_nth_page($i)->show_all;
+ for ( 1 .. $self->{notebook}->get_n_pages - 1 ) {
+  $self->{notebook}->get_nth_page($_)->show_all;
  }
 
  my $num_dev_options = $options->num_options;
- for ( my $i = 1 ; $i < $num_dev_options ; ++$i ) {
-  my $opt = $options->{array}[$i];
+ for ( 1 .. $num_dev_options - 1 ) {
+  my $opt = $options->{array}[$_];
   my $show;
   if ( defined $visible_options->{ $opt->{name} } ) {
    $show = $visible_options->{ $opt->{name} };
@@ -482,28 +494,28 @@ sub _update_option_visibility {
   }
   my $container =
     $opt->{type} == SANE_TYPE_GROUP ? $opt->{widget} : $opt->{widget}->parent;
-  my $geometry = _geometry_option($opt);
+  my $geometry = $self->_geometry_option($opt);
   if ($show) {
    $container->show_all;
 
    # Find associated group
    next if ( $opt->{type} == SANE_TYPE_GROUP );
-   my $j = $i;
+   my $j = $_;
    while ( --$j > 0 and $options->{array}[$j]{type} != SANE_TYPE_GROUP ) {
    }
    if ( $j > 0 and not $options->{array}[$j]{widget}->visible ) {
     my $group = $options->{array}[$j]{widget};
-    unless ( $group->visible ) {
+    if ( not $group->visible ) {
      $group->remove($container);
      my $move_paper =
        (    $geometry
         and defined( $self->{hboxp} )
         and $self->{hboxp}->parent eq $group );
-     $group->remove( $self->{hboxp} ) if ($move_paper);
+     if ($move_paper) { $group->remove( $self->{hboxp} ) }
 
      # Find visible group
      $group = $self->_find_visible_group( $options, $j );
-     $group->pack_start( $self->{hboxp}, FALSE, FALSE, 0 ) if ($move_paper);
+     if ($move_paper) { $group->pack_start( $self->{hboxp}, FALSE, FALSE, 0 ) }
      $group->pack_start( $container, FALSE, FALSE, 0 );
     }
    }
@@ -538,12 +550,12 @@ sub _find_visible_group {
 # Return true if we have a valid geometry option
 
 sub _geometry_option {
- my ($opt) = @_;
+ my ( $self, $opt ) = @_;
  return (
         ( $opt->{type} == SANE_TYPE_FIXED or $opt->{type} == SANE_TYPE_INT )
     and ( $opt->{unit} == SANE_UNIT_MM or $opt->{unit} == SANE_UNIT_PIXEL )
     and ( $opt->{name} =~
-   /^(?:l|t|x|y|$SANE_NAME_PAGE_HEIGHT|$SANE_NAME_PAGE_WIDTH)$/x )
+   /^(?:l|t|x|y|$SANE_NAME_PAGE_HEIGHT|$SANE_NAME_PAGE_WIDTH)$/xsm )
  );
 }
 
@@ -563,7 +575,7 @@ sub _geometry_widgets_created {
  );
 }
 
-sub _create_paper_widget {
+sub create_paper_widget {
  my ( $self, $options ) = @_;
 
  # Only define the paper size once the rest of the geometry widgets
@@ -590,7 +602,7 @@ sub _create_paper_widget {
     elsif ( $self->{combobp}->get_active_text eq $d->get('Manual') ) {
      for ( ( 'l', 't', 'x', 'y', SANE_NAME_PAGE_HEIGHT, SANE_NAME_PAGE_WIDTH ) )
      {
-      $options->{box}{$_}->show_all if ( defined $options->{box}{$_} );
+      if ( defined $options->{box}{$_} ) { $options->{box}{$_}->show_all }
      }
     }
     else {
@@ -616,15 +628,7 @@ sub _create_paper_widget {
      for (qw( l t x y )) {
       $options->by_name($_)->{widget}->set_value( $formats->{$paper}{$_} );
      }
-     Glib::Idle->add(
-      sub {
-       for (
-        ( 'l', 't', 'x', 'y', SANE_NAME_PAGE_HEIGHT, SANE_NAME_PAGE_WIDTH ) )
-       {
-        $options->{box}{$_}->hide_all if ( defined $options->{box}{$_} );
-       }
-      }
-     );
+     Glib::Idle->add( sub { $self->hide_geometry($options) } );
 
      # Do this last, as it fires the changed-paper signal
      $self->set( 'paper', $paper );
@@ -635,28 +639,10 @@ sub _create_paper_widget {
  return;
 }
 
-sub _pack_widget {
- my ( $self,    $widget, $data ) = @_;
- my ( $options, $opt,    $hbox ) = @$data;
- if ( defined $widget ) {
-  $opt->{widget} = $widget;
-  if ( $opt->{type} == SANE_TYPE_BUTTON or $opt->{max_values} > 1 ) {
-   $hbox->pack_end( $widget, TRUE, TRUE, 0 );
-  }
-  else {
-   $hbox->pack_end( $widget, FALSE, FALSE, 0 );
-  }
-  $tooltips->set_tip( $widget, $d_sane->get( $opt->{desc} ) );
-
-  # Look-up to hide/show the box if necessary
-  $options->{box}{ $opt->{name} } = $hbox
-    if ( _geometry_option($opt) );
-
-  $self->_create_paper_widget($options);
-
- }
- else {
-  $logger->warn("Unknown type $opt->{type}");
+sub hide_geometry {
+ my ( $self, $options ) = @_;
+ for ( ( 'l', 't', 'x', 'y', SANE_NAME_PAGE_HEIGHT, SANE_NAME_PAGE_WIDTH ) ) {
+  if ( defined $options->{box}{$_} ) { $options->{box}{$_}->hide_all; }
  }
  return;
 }
@@ -673,31 +659,21 @@ sub set_option {
  my $current = $self->{current_scan_options};
 
  # Cache option
- push @$current, { $option->{name} => $val };
+ push @{$current}, { $option->{name} => $val };
 
  # Note any duplicate options, keeping only the last entry.
- my %seen;
-
- my $j = $#{$current};
- while ( $j > -1 ) {
-  my ($opt) =
-    keys( %{ $current->[$j] } );
-  $seen{$opt}++;
-  if ( $seen{$opt} > 1 ) {
-   splice @$current, $j, 1;
-  }
-  $j--;
- }
- $self->{current_scan_options} = $current;
+ $self->{current_scan_options} =
+   Gscan2pdf::Scanner::Options::prune_duplicates($current);
 
  # Do we need to reload?
  my $reload_triggers = $self->get('reload-triggers');
  my $reload_flag;
  if ( defined $reload_triggers ) {
-  $reload_triggers = [$reload_triggers]
-    if ( ref($reload_triggers) ne 'ARRAY' );
+  if ( ref($reload_triggers) ne 'ARRAY' ) {
+   $reload_triggers = [$reload_triggers];
+  }
 
-  for (@$reload_triggers) {
+  for ( @{$reload_triggers} ) {
    if ( $_ eq $option->{name} or $_ eq $option->{title} ) {
     $reload_flag = TRUE;
     last;
@@ -708,7 +684,7 @@ sub set_option {
 
   # Try to reload from the cache
   $reload_flag = FALSE;
-  my $cache_key = '';
+  my $cache_key = $EMPTY;
   if ( $self->get('cache-options') ) {
    $cache_key = $self->cache_key();
 
@@ -721,12 +697,12 @@ sub set_option {
      $cache_key );
     $logger->info($options);
 
-    $self->update_options( $options, $option ) if ($options);
+    if ($options) { $self->update_options( $options, $option ) }
 
     $self->signal_emit( 'finished-process', 'find_scan_options' );
 
     # Unset the profile unless we are actively setting it
-    $self->set( 'profile', undef ) unless ( $self->{setting_profile} );
+    if ( not $self->{setting_profile} ) { $self->set( 'profile', undef ) }
 
     $self->signal_emit( 'changed-scan-option', $option->{name}, $val );
     $self->signal_emit('reloaded-scan-options');
@@ -734,7 +710,7 @@ sub set_option {
   }
 
   # Reload from the scanner
-  unless ($reload_flag) {
+  if ( not $reload_flag ) {
    my $pbar;
    my $hboxd = $self->{hboxd};
    Gscan2pdf::Frontend::CLI->find_scan_options(
@@ -746,7 +722,7 @@ sub set_option {
 
      # Set up ProgressBar
      $pbar = Gtk2::ProgressBar->new;
-     $pbar->set_pulse_step(.1);
+     $pbar->set_pulse_step( $self->get('progress-pulse-step') );
      $pbar->set_text( $d->get('Updating options') );
      $hboxd->pack_start( $pbar, TRUE, TRUE, 0 );
      $hboxd->hide_all;
@@ -777,12 +753,12 @@ sub set_option {
       }
       $self->signal_emit( 'changed-options-cache', $cache );
      }
-     $self->update_options($options) if ($options);
+     if ($options) { $self->update_options($options) }
 
      $self->signal_emit( 'finished-process', 'find_scan_options' );
 
      # Unset the profile unless we are actively setting it
-     $self->set( 'profile', undef ) unless ( $self->{setting_profile} );
+     if ( not $self->{setting_profile} ) { $self->set( 'profile', undef ) }
 
      $self->signal_emit( 'changed-scan-option', $option->{name}, $val );
      $self->signal_emit('reloaded-scan-options');
@@ -798,7 +774,7 @@ sub set_option {
  }
 
  # Unset the profile unless we are actively setting it
- $self->set( 'profile', undef ) unless ( $self->{setting_profile} );
+ if ( not $self->{setting_profile} ) { $self->set( 'profile', undef ) }
 
  $self->signal_emit( 'changed-scan-option', $option->{name}, $val );
  return;
@@ -825,19 +801,21 @@ sub update_widget {
    # CheckButton
    if ( $opt->{type} == SANE_TYPE_BOOL )
    {    ## no critic (ProhibitCascadingIfElse)
-    $widget->set_active($value)
-      if ( $self->value_for_active_option( $value, $opt ) );
+    if ( $self->value_for_active_option( $value, $opt ) ) {
+     $widget->set_active($value);
+    }
    }
 
    # SpinButton
    elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_RANGE ) {
     my ( $step, $page ) = $widget->get_increments;
     $step = 1;
-    $step = $opt->{constraint}{quant} if ( $opt->{constraint}{quant} );
+    if ( $opt->{constraint}{quant} ) { $step = $opt->{constraint}{quant} }
     $widget->set_range( $opt->{constraint}{min}, $opt->{constraint}{max} );
     $widget->set_increments( $step, $page );
-    $widget->set_value($value)
-      if ( $self->value_for_active_option( $value, $opt ) );
+    if ( $self->value_for_active_option( $value, $opt ) ) {
+     $widget->set_value($value);
+    }
    }
 
    # ComboBox
@@ -846,17 +824,18 @@ sub update_widget {
    {
     $widget->get_model->clear;
     my $index = 0;
-    for ( my $i = 0 ; $i < @{ $opt->{constraint} } ; ++$i ) {
-     $widget->append_text( $d_sane->get( $opt->{constraint}[$i] ) );
-     $index = $i if ( defined $value and $opt->{constraint}[$i] eq $value );
+    for ( 0 .. $#{ $opt->{constraint} } ) {
+     $widget->append_text( $d_sane->get( $opt->{constraint}[$_] ) );
+     if ( defined $value and $opt->{constraint}[$_] eq $value ) { $index = $_ }
     }
-    $widget->set_active($index) if ( defined $index );
+    if ( defined $index ) { $widget->set_active($index) }
    }
 
    # Entry
    elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_NONE ) {
-    $widget->set_text($value)
-      if ( $self->value_for_active_option( $value, $opt ) );
+    if ( $self->value_for_active_option( $value, $opt ) ) {
+     $widget->set_text($value);
+    }
    }
   }
   $widget->signal_handler_unblock( $widget->{signal} );
@@ -874,9 +853,9 @@ sub update_options {
 
  my ( $group, $vbox );
  my $num_dev_options = $options->num_options;
- for ( my $i = 1 ; $i < $num_dev_options ; ++$i ) {
-  my $opt = $options->by_index($i);
-  if ( defined( $opt->{name} ) and $opt->{name} ne '' ) {
+ for ( 1 .. $num_dev_options - 1 ) {
+  my $opt = $options->by_index($_);
+  if ( defined( $opt->{name} ) and $opt->{name} ne $EMPTY ) {
 
    # If we are loading from the cache, then both the current options,
    # and the widgets could be different
@@ -893,12 +872,14 @@ sub update_options {
 
      # Don't update if the old option is still available
      my $val = quotemeta( $old_opt->{val} ); # quote any possible metacharacters
-     unless (
-      (
-          $opt->{constraint_type} == SANE_CONSTRAINT_STRING_LIST
-       or $opt->{constraint_type} == SANE_CONSTRAINT_WORD_LIST
+     if (
+      not(
+       (
+           $opt->{constraint_type} == SANE_CONSTRAINT_STRING_LIST
+        or $opt->{constraint_type} == SANE_CONSTRAINT_WORD_LIST
+       )
+       and any { /^$val$/xsm } @{ $opt->{constraint} }
       )
-      and grep { /^$val$/x } @{ $opt->{constraint} }
        )
      {
       $self->set_option( $old_opt, $opt->{val} );
@@ -918,7 +899,7 @@ sub update_options {
 sub set_current_scan_options {
  my ( $self, $profile ) = @_;
 
- return unless ( defined $profile );
+ if ( not defined($profile) ) { return }
 
  # As scanimage and scanadf rename the geometry options,
  # we have to map them back to the original names
@@ -930,10 +911,10 @@ sub set_current_scan_options {
  # Config::General flattens arrays with 1 entry to scalars,
  # so we must check for this
  if ( ref($profile) ne 'ARRAY' ) {
-  push @$defaults, $profile;
+  push @{$defaults}, $profile;
  }
  else {
-  @$defaults = @$profile;
+  @{$defaults} = @{$profile};
  }
 
  # Give the GUI a chance to catch up between settings,
@@ -988,7 +969,7 @@ sub set_current_scan_options {
 sub set_option_widget {
  my ( $self, $i, $profile ) = @_;
 
- while ( $i < @$profile ) {
+ while ( $i < @{$profile} ) {
 
   # for reasons I don't understand, without walking the reference tree,
   # parts of $profile are undef
@@ -1006,7 +987,7 @@ sub set_option_widget {
 
   # If the option is inactive, then remove it from the profile
   if ( $opt->{cap} & SANE_CAP_INACTIVE ) {
-   splice @$profile, $i, 1;
+   splice @{$profile}, $i, 1;
    $self->{current_scan_options} = $profile;
    return $i;
   }
@@ -1020,7 +1001,7 @@ sub set_option_widget {
   else {
    given ($widget) {
     when ( $widget->isa('Gtk2::CheckButton') ) {
-     $val = SANE_FALSE if ( $val eq '' );
+     if ( $val eq $EMPTY ) { $val = SANE_FALSE }
      if ( $widget->get_active != $val ) {
       $widget->set_active($val);
       return $i;
@@ -1035,10 +1016,10 @@ sub set_option_widget {
     when ( $widget->isa('Gtk2::ComboBox') ) {
      if ( $opt->{constraint}[ $widget->get_active ] ne $val ) {
       my $index;
-      for ( my $j = 0 ; $j < @{ $opt->{constraint} } ; ++$j ) {
-       $index = $j if ( $opt->{constraint}[$j] eq $val );
+      for ( 0 .. $#{ $opt->{constraint} } ) {
+       if ( $opt->{constraint}[$_] eq $val ) { $index = $_ }
       }
-      $widget->set_active($index) if ( defined $index );
+      if ( defined $index ) { $widget->set_active($index) }
       return $i;
      }
     }
@@ -1078,17 +1059,19 @@ sub map_geometry_names {
    when (SANE_NAME_SCAN_BR_X) {
     $name = 'x';
     my $l = $self->get_option_from_profile( 'l', $profile );
-    $l = $self->get_option_from_profile( SANE_NAME_SCAN_TL_X, $profile )
-      unless ( defined $l );
-    $val -= $l if ( defined $l );
+    if ( not defined($l) ) {
+     $l = $self->get_option_from_profile( SANE_NAME_SCAN_TL_X, $profile );
+    }
+    if ( defined $l ) { $val -= $l }
     $profile->[$i] = { $name => $val };
    }
    when (SANE_NAME_SCAN_BR_Y) {
     $name = 'y';
     my $t = $self->get_option_from_profile( 't', $profile );
-    $t = $self->get_option_from_profile( SANE_NAME_SCAN_TL_Y, $profile )
-      unless ( defined $t );
-    $val -= $t if ( defined $t );
+    if ( not defined($t) ) {
+     $t = $self->get_option_from_profile( SANE_NAME_SCAN_TL_Y, $profile );
+    }
+    if ( defined $t ) { $val -= $t }
     $profile->[$i] = { $name => $val };
    }
   }
@@ -1102,18 +1085,18 @@ sub map_options {
  my ( $self, $old ) = @_;
  my $new;
  my $options = $self->get('available-scan-options');
- for (@$old) {
+ for ( @{$old} ) {
 
   # for reasons I don't understand, without walking the reference tree,
   # parts of $_ are undef
   Gscan2pdf::Dialog::Scan::my_dumper($_);
-  my ( $key, $val ) = each(%$_);
-  unless ( $key eq 'Paper size' ) {
+  my ( $key, $val ) = each( %{$_} );
+  if ( $key ne 'Paper size' ) {
    my $opt = $options->by_name($key);
    if ( defined( $opt->{type} ) and $opt->{type} == SANE_TYPE_BOOL ) {
     $val = $val ? 'yes' : 'no';
    }
-   push @$new, { $key => $val };
+   push @{$new}, { $key => $val };
   }
  }
  return $new;
@@ -1126,8 +1109,7 @@ sub scan {
  my $npages = $self->get('num-pages');
  my $start  = $self->get('page-number-start');
  my $step   = $self->get('page-number-increment');
- $npages = $self->get('max-pages')
-   if ( $npages > 0 and $step < 0 );
+ if ( $npages > 0 and $step < 0 ) { $npages = $self->get('max-pages') }
 
  if ( $start == 1 and $step < 0 ) {
   $self->signal_emit( 'process-error', 'scan',
