@@ -20,6 +20,8 @@ use Glib::Object::Subclass Gscan2pdf::Dialog::Scan::, properties => [
  ),
 ];
 
+our $VERSION = '1.2.3';
+
 my $SANE_NAME_SCAN_TL_X   = SANE_NAME_SCAN_TL_X;
 my $SANE_NAME_SCAN_TL_Y   = SANE_NAME_SCAN_TL_Y;
 my $SANE_NAME_SCAN_BR_X   = SANE_NAME_SCAN_BR_X;
@@ -46,7 +48,10 @@ sub SET_PROPERTY {
  if (( defined($newval) and defined($oldval) and $newval ne $oldval )
   or ( defined($newval) xor defined($oldval) ) )
  {
-  if ( $name eq 'logger' ) { $logger = $self->get('logger') }
+  if ( $name eq 'logger' ) {
+   $logger = $newval;
+   $logger->debug('Set logger in Gscan2pdf::Dialog::Scan::Sane');
+  }
  }
  $self->SUPER::SET_PROPERTY( $pspec, $newval );
  return;
@@ -64,7 +69,7 @@ sub get_devices {
 
    # Set up ProgressBar
    $pbar = Gtk2::ProgressBar->new;
-   $pbar->set_pulse_step(.1);
+   $pbar->set_pulse_step( $self->get('progress-pulse-step') );
    $pbar->set_text( $d->get('Fetching list of devices') );
    $hboxd->pack_start( $pbar, TRUE, TRUE, 0 );
    $hboxd->hide_all;
@@ -101,11 +106,13 @@ sub scan_options {
 
  # Remove any existing pages
  while ( $self->{notebook}->get_n_pages > 1 ) {
-  $self->{notebook}->remove_page(-1);
+
+  # -1 = last page
+  $self->{notebook}->remove_page(-1);    ## no critic (ProhibitMagicNumbers)
  }
 
  # Ghost the scan button whilst options being updated
- $self->{sbutton}->set_sensitive(FALSE) if ( defined $self->{sbutton} );
+ if ( defined $self->{sbutton} ) { $self->{sbutton}->set_sensitive(FALSE) }
 
  my $signal;
  Gscan2pdf::Frontend::Sane->open_device(
@@ -114,7 +121,7 @@ sub scan_options {
    $self->signal_emit( 'started-process', $d->get('Opening device') );
   },
   running_callback => sub {
-   $self->signal_emit( 'changed-progress', -1, undef );
+   $self->signal_emit( 'changed-progress', undef, undef );
   },
   finished_callback => sub {
    $self->signal_emit( 'finished-process', 'open_device' );
@@ -123,7 +130,7 @@ sub scan_options {
      $self->signal_emit( 'started-process', $d->get('Retrieving options') );
     },
     sub {    # running callback
-     $self->signal_emit( 'changed-progress', -1, undef );
+     $self->signal_emit( 'changed-progress', undef, undef );
     },
     sub {    # finished callback
      my ($data) = @_;
@@ -151,7 +158,6 @@ sub scan_options {
    $self->destroy;
   }
  );
-
  return;
 }
 
@@ -169,8 +175,8 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
  $self->{paper}         = undef;
 
  delete $self->{combobp};    # So we don't carry over from one device to another
- for ( my $i = 1 ; $i < $num_dev_options ; ++$i ) {
-  my $opt = $options->by_index($i);
+ for ( 1 .. $num_dev_options - 1 ) {
+  my $opt = $options->by_index($_);
 
   # Notebook page for group
   if ( $opt->{type} == SANE_TYPE_GROUP or not defined($vbox) ) {
@@ -183,7 +189,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
    next;
   }
 
-  next unless ( $opt->{cap} & SANE_CAP_SOFT_DETECT );
+  if ( not( $opt->{cap} & SANE_CAP_SOFT_DETECT ) ) { next }
 
   # Widget
   my ( $widget, $val );
@@ -191,7 +197,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
 
   # Define HBox for paper size here
   # so that it can be put before first geometry option
-  if ( _geometry_option($opt) and not defined($hboxp) ) {
+  if ( $self->_geometry_option($opt) and not defined($hboxp) ) {
    $hboxp = Gtk2::HBox->new;
    $vbox->pack_start( $hboxp, FALSE, FALSE, 0 );
   }
@@ -199,9 +205,11 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
   # HBox for option
   my $hbox = Gtk2::HBox->new;
   $vbox->pack_start( $hbox, FALSE, TRUE, 0 );
-  $hbox->set_sensitive(FALSE)
-    if ( $opt->{cap} & SANE_CAP_INACTIVE
-   or not $opt->{cap} & SANE_CAP_SOFT_SELECT );
+  if ( $opt->{cap} & SANE_CAP_INACTIVE
+   or not $opt->{cap} & SANE_CAP_SOFT_SELECT )
+  {
+   $hbox->set_sensitive(FALSE);
+  }
 
   if ( $opt->{max_values} < 2 ) {
 
@@ -215,7 +223,7 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
    if ( $opt->{type} == SANE_TYPE_BOOL )
    {    ## no critic (ProhibitCascadingIfElse)
     $widget = Gtk2::CheckButton->new;
-    $widget->set_active(TRUE) if ($val);
+    if ($val) { $widget->set_active(TRUE) }
     $widget->{signal} = $widget->signal_connect(
      toggled => sub {
       my $value = $widget->get_active;
@@ -237,13 +245,14 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
    # SpinButton
    elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_RANGE ) {
     my $step = 1;
-    $step = $opt->{constraint}{quant} if ( $opt->{constraint}{quant} );
+    if ( $opt->{constraint}{quant} ) { $step = $opt->{constraint}{quant} }
     $widget = Gtk2::SpinButton->new_with_range( $opt->{constraint}{min},
      $opt->{constraint}{max}, $step );
 
     # Set the default
-    $widget->set_value($val)
-      if ( defined $val and not $opt->{cap} & SANE_CAP_INACTIVE );
+    if ( defined $val and not $opt->{cap} & SANE_CAP_INACTIVE ) {
+     $widget->set_value($val);
+    }
     $widget->{signal} = $widget->signal_connect(
      'value-changed' => sub {
       my $value = $widget->get_value;
@@ -258,13 +267,13 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
    {
     $widget = Gtk2::ComboBox->new_text;
     my $index = 0;
-    for ( my $i = 0 ; $i < @{ $opt->{constraint} } ; ++$i ) {
-     $widget->append_text( $d_sane->get( $opt->{constraint}[$i] ) );
-     $index = $i if ( defined $val and $opt->{constraint}[$i] eq $val );
+    for ( 0 .. $#{ $opt->{constraint} } ) {
+     $widget->append_text( $d_sane->get( $opt->{constraint}[$_] ) );
+     if ( defined $val and $opt->{constraint}[$_] eq $val ) { $index = $_ }
     }
 
     # Set the default
-    $widget->set_active($index) if ( defined $index );
+    if ( defined $index ) { $widget->set_active($index) }
     $widget->{signal} = $widget->signal_connect(
      changed => sub {
       my $i = $widget->get_active;
@@ -278,8 +287,9 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
     $widget = Gtk2::Entry->new;
 
     # Set the default
-    $widget->set_text($val)
-      if ( defined $val and not $opt->{cap} & SANE_CAP_INACTIVE );
+    if ( defined $val and not $opt->{cap} & SANE_CAP_INACTIVE ) {
+     $widget->set_text($val);
+    }
     $widget->{signal} = $widget->signal_connect(
      activate => sub {
       my $value = $widget->get_text;
@@ -296,15 +306,15 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
    );
   }
 
-  $self->_pack_widget( $widget, [ $options, $opt, $hbox, $hboxp ] );
+  $self->pack_widget( $widget, [ $options, $opt, $hbox, $hboxp ] );
  }
 
  # Set defaults
  my $sane_device = Gscan2pdf::Frontend::Sane->device();
 
  # Show new pages
- for ( my $i = 1 ; $i < $self->{notebook}->get_n_pages ; $i++ ) {
-  $self->{notebook}->get_nth_page($i)->show_all;
+ for ( 1 .. $self->{notebook}->get_n_pages - 1 ) {
+  $self->{notebook}->get_nth_page($_)->show_all;
  }
 
  $self->{sbutton}->set_sensitive(TRUE);
@@ -315,16 +325,17 @@ sub _initialise_options {    ## no critic (ProhibitExcessComplexity)
 # Return true if we have a valid geometry option
 
 sub _geometry_option {
- my ($opt) = @_;
- return
-       ( $opt->{type} == SANE_TYPE_FIXED or $opt->{type} == SANE_TYPE_INT )
-   and ( $opt->{unit} == SANE_UNIT_MM or $opt->{unit} == SANE_UNIT_PIXEL )
-   and ( $opt->{name} =~
-/^(?:$SANE_NAME_SCAN_TL_X|$SANE_NAME_SCAN_TL_Y|$SANE_NAME_SCAN_BR_X|$SANE_NAME_SCAN_BR_Y|$SANE_NAME_PAGE_HEIGHT|$SANE_NAME_PAGE_WIDTH)$/x
-   );
+ my ( $self, $opt ) = @_;
+ return (
+        ( $opt->{type} == SANE_TYPE_FIXED or $opt->{type} == SANE_TYPE_INT )
+    and ( $opt->{unit} == SANE_UNIT_MM or $opt->{unit} == SANE_UNIT_PIXEL )
+    and ( $opt->{name} =~
+/^(?:$SANE_NAME_SCAN_TL_X|$SANE_NAME_SCAN_TL_Y|$SANE_NAME_SCAN_BR_X|$SANE_NAME_SCAN_BR_Y|$SANE_NAME_PAGE_HEIGHT|$SANE_NAME_PAGE_WIDTH)$/xms
+    )
+ );
 }
 
-sub _create_paper_widget {
+sub create_paper_widget {
  my ( $self, $options, $hboxp ) = @_;
 
  # Only define the paper size once the rest of the geometry widgets
@@ -367,7 +378,7 @@ sub _create_paper_widget {
       )
        )
      {
-      $options->{box}{$_}->show_all if ( defined $options->{box}{$_} );
+      if ( defined $options->{box}{$_} ) { $options->{box}{$_}->show_all; }
      }
     }
     else {
@@ -390,19 +401,7 @@ sub _create_paper_widget {
        ->set_value( $formats->{$paper}{x} + $formats->{$paper}{l} );
      $options->by_name(SANE_NAME_SCAN_BR_Y)->{widget}
        ->set_value( $formats->{$paper}{y} + $formats->{$paper}{t} );
-     Glib::Idle->add(
-      sub {
-       for (
-        ( SANE_NAME_SCAN_TL_X, SANE_NAME_SCAN_TL_Y,
-         SANE_NAME_SCAN_BR_X,   SANE_NAME_SCAN_BR_Y,
-         SANE_NAME_PAGE_HEIGHT, SANE_NAME_PAGE_WIDTH
-        )
-         )
-       {
-        $options->{box}{$_}->hide_all if ( defined $options->{box}{$_} );
-       }
-      }
-     );
+     Glib::Idle->add( sub { $self->hide_geometry($options) } );
 
      # Do this last, as it fires the changed-paper signal
      $self->set( 'paper', $paper );
@@ -413,28 +412,16 @@ sub _create_paper_widget {
  return;
 }
 
-sub _pack_widget {
- my ( $self, $widget, $data ) = @_;
- my ( $options, $opt, $hbox, $hboxp ) = @$data;
- if ( defined $widget ) {
-  $opt->{widget} = $widget;
-  if ( $opt->{type} == SANE_TYPE_BUTTON or $opt->{max_values} > 1 ) {
-   $hbox->pack_end( $widget, TRUE, TRUE, 0 );
-  }
-  else {
-   $hbox->pack_end( $widget, FALSE, FALSE, 0 );
-  }
-  $tooltips->set_tip( $widget, $d_sane->get( $opt->{desc} ) );
-
-  # Look-up to hide/show the box if necessary
-  $options->{box}{ $opt->{name} } = $hbox
-    if ( _geometry_option($opt) );
-
-  $self->_create_paper_widget( $options, $hboxp );
-
- }
- else {
-  $logger->warn("Unknown type $opt->{type}");
+sub hide_geometry {
+ my ( $self, $options ) = @_;
+ for (
+  ( SANE_NAME_SCAN_TL_X, SANE_NAME_SCAN_TL_Y,
+   SANE_NAME_SCAN_BR_X,   SANE_NAME_SCAN_BR_Y,
+   SANE_NAME_PAGE_HEIGHT, SANE_NAME_PAGE_WIDTH
+  )
+   )
+ {
+  if ( defined $options->{box}{$_} ) { $options->{box}{$_}->hide_all; }
  }
  return;
 }
@@ -449,22 +436,11 @@ sub set_option {
  my $current = $self->{current_scan_options};
 
  # Cache option
- push @$current, { $option->{name} => $val };
+ push @{$current}, { $option->{name} => $val };
 
  # Note any duplicate options, keeping only the last entry.
- my %seen;
-
- my $j = $#{$current};
- while ( $j > -1 ) {
-  my ($opt) =
-    keys( %{ $current->[$j] } );
-  $seen{$opt}++;
-  if ( $seen{$opt} > 1 ) {
-   splice @$current, $j, 1;
-  }
-  $j--;
- }
- $self->{current_scan_options} = $current;
+ $self->{current_scan_options} =
+   Gscan2pdf::Scanner::Options::prune_duplicates($current);
 
  my $signal;
  my $options = $self->get('available-scan-options');
@@ -476,18 +452,19 @@ sub set_option {
     $option->{name} );
   },
   running_callback => sub {
-   $self->signal_emit( 'changed-progress', -1, undef );
+   $self->signal_emit( 'changed-progress', undef, undef );
   },
   finished_callback => sub {
    my ($data) = @_;
-   $self->update_options( Gscan2pdf::Scanner::Options->new_from_data($data) )
-     if ($data);
+   if ($data) {
+    $self->update_options( Gscan2pdf::Scanner::Options->new_from_data($data) );
+   }
 
    # We can carry on applying defaults now, if necessary.
    $self->signal_emit( 'finished-process', 'set_option' );
 
    # Unset the profile unless we are actively setting it
-   $self->set( 'profile', undef ) unless ( $self->{setting_profile} );
+   if ( not $self->{setting_profile} ) { $self->set( 'profile', undef ); }
 
    $self->signal_emit( 'changed-scan-option', $option->{name}, $val );
 
@@ -501,68 +478,86 @@ sub set_option {
 # If setting an option triggers a reload, we need to update the options
 
 sub update_options {
- my ( $self, $options ) = @_;
+ my ( $self, $new_options ) = @_;
 
  # walk the widget tree and update them from the hash
- $logger->debug( "Sane->get_option_descriptor returned: ", Dumper($options) );
+ $logger->debug( "Sane->get_option_descriptor returned: ",
+  Dumper($new_options) );
 
  my ( $group, $vbox );
- my $num_dev_options = $options->num_options;
- for ( my $i = 1 ; $i < $num_dev_options ; ++$i ) {
-  my $widget = $self->get('available-scan-options')->by_index($i)->{widget};
+ my $num_dev_options = $new_options->num_options;
+ my $options         = $self->get('available-scan-options');
+ for ( 1 .. $num_dev_options - 1 ) {
+  my $widget = $options->by_index($_)->{widget};
 
-  # could be undefined for !($opt->{cap} & SANE_CAP_SOFT_DETECT)
-  if ( defined $widget ) {
-   my $opt   = $options->by_index($i);
-   my $value = $opt->{val};
-   $widget->signal_handler_block( $widget->{signal} );
+  # could be undefined for !($new_opt->{cap} & SANE_CAP_SOFT_DETECT)
+  if ( not defined($widget) ) { next; }
 
-   # HBox for option
-   my $hbox = $widget->parent;
-   $hbox->set_sensitive( ( not $opt->{cap} & SANE_CAP_INACTIVE )
-      and $opt->{cap} & SANE_CAP_SOFT_SELECT );
+  my $new_opt = $new_options->by_index($_);
+  my $opt     = $options->by_index($_);
+  if ( $new_opt->{name} ne $opt->{name} ) {
+   $logger->error(
+    'Error updating options: reloaded options are numbered differently');
+   return;
+  }
 
-   if ( $opt->{max_values} < 2 ) {
+  # We've got to block the signal handler for the widget to prevent infinite
+  # loops of the widget updating the option, updating the widget, etc., but
+  # therefore we must update the options manually
+  for (qw(val cap max_values type constraint_type constraint)) {
+   $opt->{$_} = $new_opt->{$_};
+  }
+  $widget->signal_handler_block( $widget->{signal} );
+  my $value = $opt->{val};
 
-    # CheckButton
-    if ( $opt->{type} == SANE_TYPE_BOOL )
-    {    ## no critic (ProhibitCascadingIfElse)
-     $widget->set_active($value)
-       if ( $self->value_for_active_option( $value, $opt ) );
-    }
+  # HBox for option
+  my $hbox = $widget->parent;
+  $hbox->set_sensitive( ( not $opt->{cap} & SANE_CAP_INACTIVE )
+     and $opt->{cap} & SANE_CAP_SOFT_SELECT );
 
-    # SpinButton
-    elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_RANGE ) {
-     my ( $step, $page ) = $widget->get_increments;
-     $step = 1;
-     $step = $opt->{constraint}{quant} if ( $opt->{constraint}{quant} );
-     $widget->set_range( $opt->{constraint}{min}, $opt->{constraint}{max} );
-     $widget->set_increments( $step, $page );
-     $widget->set_value($value)
-       if ( $self->value_for_active_option( $value, $opt ) );
-    }
+  if ( $opt->{max_values} < 2 ) {
 
-    # ComboBox
-    elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_STRING_LIST
-     or $opt->{constraint_type} == SANE_CONSTRAINT_WORD_LIST )
-    {
-     $widget->get_model->clear;
-     my $index = 0;
-     for ( my $i = 0 ; $i < @{ $opt->{constraint} } ; ++$i ) {
-      $widget->append_text( $d_sane->get( $opt->{constraint}[$i] ) );
-      $index = $i if ( defined $value and $opt->{constraint}[$i] eq $value );
-     }
-     $widget->set_active($index) if ( defined $index );
+   # CheckButton
+   if ( $opt->{type} == SANE_TYPE_BOOL )
+   {    ## no critic (ProhibitCascadingIfElse)
+    if ( $self->value_for_active_option( $value, $opt ) ) {
+     $widget->set_active($value);
     }
+   }
 
-    # Entry
-    elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_NONE ) {
-     $widget->set_text($value)
-       if ( $self->value_for_active_option( $value, $opt ) );
+   # SpinButton
+   elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_RANGE ) {
+    my ( $step, $page ) = $widget->get_increments;
+    $step = 1;
+    if ( $opt->{constraint}{quant} ) { $step = $opt->{constraint}{quant} }
+    $widget->set_range( $opt->{constraint}{min}, $opt->{constraint}{max} );
+    $widget->set_increments( $step, $page );
+    if ( $self->value_for_active_option( $value, $opt ) ) {
+     $widget->set_value($value);
+    }
+   }
+
+   # ComboBox
+   elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_STRING_LIST
+    or $opt->{constraint_type} == SANE_CONSTRAINT_WORD_LIST )
+   {
+    $widget->get_model->clear;
+    my $index = 0;
+    for ( 0 .. $#{ $opt->{constraint} } ) {
+     $widget->append_text( $d_sane->get( $opt->{constraint}[$_] ) );
+     if ( defined $value and $opt->{constraint}[$_] eq $value ) { $index = $_ }
+    }
+    if ( defined $index ) { $widget->set_active($index) }
+   }
+
+   # Entry
+   elsif ( $opt->{constraint_type} == SANE_CONSTRAINT_NONE ) {
+    if ( $self->value_for_active_option( $value, $opt ) ) {
+     $widget->set_text($value);
     }
    }
-   $widget->signal_handler_unblock( $widget->{signal} );
   }
+  $widget->signal_handler_unblock( $widget->{signal} );
  }
  return;
 }
@@ -572,7 +567,7 @@ sub update_options {
 sub set_current_scan_options {
  my ( $self, $profile ) = @_;
 
- return unless ( defined $profile );
+ if ( not defined($profile) ) { return }
 
  # Move them first to a dummy array, as otherwise it would be self-modifying
  my $defaults;
@@ -580,10 +575,10 @@ sub set_current_scan_options {
  # Config::General flattens arrays with 1 entry to scalars,
  # so we must check for this
  if ( ref($profile) ne 'ARRAY' ) {
-  push @$defaults, $profile;
+  push @{$defaults}, $profile;
  }
  else {
-  @$defaults = @$profile;
+  @{$defaults} = @{$profile};
  }
 
  # Give the GUI a chance to catch up between settings,
@@ -638,7 +633,7 @@ sub set_current_scan_options {
 sub set_option_widget {
  my ( $self, $i, $profile ) = @_;
 
- while ( $i < @$profile ) {
+ while ( $i < @{$profile} ) {
 
   # for reasons I don't understand, without walking the reference tree,
   # parts of $profile are undef
@@ -664,17 +659,19 @@ sub set_option_widget {
    when ('x') {
     $name = SANE_NAME_SCAN_BR_X;
     my $l = $self->get_option_from_profile( 'l', $profile );
-    $l = $self->get_option_from_profile( SANE_NAME_SCAN_TL_X, $profile )
-      unless ( defined $l );
-    $val += $l if ( defined $l );
+    if ( not defined($l) ) {
+     $l = $self->get_option_from_profile( SANE_NAME_SCAN_TL_X, $profile );
+    }
+    if ( defined $l ) { $val += $l; }
     $profile->[$i] = { $name => $val };
    }
    when ('y') {
     $name = SANE_NAME_SCAN_BR_Y;
     my $t = $self->get_option_from_profile( 't', $profile );
-    $t = $self->get_option_from_profile( SANE_NAME_SCAN_TL_Y, $profile )
-      unless ( defined $t );
-    $val += $t if ( defined $t );
+    if ( not defined($t) ) {
+     $t = $self->get_option_from_profile( SANE_NAME_SCAN_TL_Y, $profile );
+    }
+    if ( defined $t ) { $val += $t; }
     $profile->[$i] = { $name => $val };
    }
   }
@@ -707,10 +704,10 @@ sub set_option_widget {
     when ( $widget->isa('Gtk2::ComboBox') ) {
      if ( $opt->{constraint}[ $widget->get_active ] ne $val ) {
       my $index;
-      for ( my $j = 0 ; $j < @{ $opt->{constraint} } ; ++$j ) {
-       $index = $j if ( $opt->{constraint}[$j] eq $val );
+      for ( 0 .. $#{ $opt->{constraint} } ) {
+       if ( $opt->{constraint}[$_] eq $val ) { $index = $_ }
       }
-      $widget->set_active($index) if ( defined $index );
+      if ( defined $index ) { $widget->set_active($index) }
       return $i;
      }
     }
@@ -733,15 +730,9 @@ sub scan {
 
  # Get selected number of pages
  my $npages = $self->get('num-pages');
-
- # Gscan2pdf::Frontend::Sane uses -1 for all
- # Gscan2pdf::Dialog::Sane uses 0 for all, as -1 puts 999 in spinbox
- $npages = -1 if ( $npages == 0 );
-
- my $start = $self->get('page-number-start');
- my $step  = $self->get('page-number-increment');
- $npages = $self->get('max-pages')
-   if ( $npages > 0 and $step < 0 );
+ my $start  = $self->get('page-number-start');
+ my $step   = $self->get('page-number-increment');
+ if ( $npages > 0 and $step < 0 ) { $npages = $self->get('max-pages') }
 
  if ( $start == 1 and $step < 0 ) {
   $self->signal_emit( 'process-error', 'scan',
