@@ -16,8 +16,16 @@ use Gscan2pdf::NetPBM;
 use Gscan2pdf::Scanner::Options;
 use Cwd;
 use File::Spec;
+use Readonly;
+Readonly my $_POLL_INTERVAL   => 100;    # ms
+Readonly my $_100             => 100;
+Readonly my $_SANE_STATUS_EOF => 5;      # or we could use Sane
+Readonly my $_1KB             => 1024;
 
-my $_POLL_INTERVAL = 100;    # ms
+our $VERSION = '1.2.3';
+
+my $EMPTY = q{};
+my $COMMA = q{,};
 my ( $_self, $logger, $d );
 
 sub setup {
@@ -37,9 +45,10 @@ sub get_devices {
   running_callback  => $options{running_callback},
   finished_callback => sub {
    my ( $output, $error ) = @_;
-   $options{finished_callback}
-     ->( Gscan2pdf::Frontend::CLI->parse_device_list($output) )
-     if ( defined $options{finished_callback} );
+   if ( defined $options{finished_callback} ) {
+    $options{finished_callback}
+      ->( Gscan2pdf::Frontend::CLI->parse_device_list($output) );
+   }
   }
  );
  return;
@@ -50,22 +59,23 @@ sub parse_device_list {
 
  my (@device_list);
 
- $logger->info($output) if ( defined($output) );
+ if ( defined($output) ) { $logger->info($output) }
 
  # parse out the device and model names
  my @words =
-   &parse_line( ',', 0, substr( $output, 0, index( $output, "'\n" ) + 1 ) );
- while ( @words == 5 ) {
+   parse_line( $COMMA, 0, substr( $output, 0, index( $output, "'\n" ) + 1 ) );
+ while (@words) {
   $output = substr( $output, index( $output, "'\n" ) + 2, length($output) );
+  shift @words;
   push @device_list,
     {
-   name   => $words[1],
-   vendor => $words[2],
-   model  => $words[3],
-   type   => $words[4]
+   name   => shift @words,
+   vendor => shift @words,
+   model  => shift @words,
+   type   => shift @words
     };
   @words =
-    &parse_line( ',', 0, substr( $output, 0, index( $output, "'\n" ) + 1 ) );
+    parse_line( $COMMA, 0, substr( $output, 0, index( $output, "'\n" ) + 1 ) );
  }
 
  return \@device_list;
@@ -74,9 +84,10 @@ sub parse_device_list {
 sub find_scan_options {
  my ( $class, %options ) = @_;
 
- $options{prefix} = '' unless ( defined $options{prefix} );
- $options{frontend} = 'scanimage'
-   if ( not defined( $options{frontend} ) or $options{frontend} eq '' );
+ if ( not defined( $options{prefix} ) ) { $options{prefix} = $EMPTY }
+ if ( not defined( $options{frontend} ) or $options{frontend} eq $EMPTY ) {
+  $options{frontend} = 'scanimage';
+ }
 
  # Get output from scanimage or scanadf.
  # Inverted commas needed for strange characters in device name
@@ -88,15 +99,17 @@ sub find_scan_options {
   running_callback  => $options{running_callback},
   finished_callback => sub {
    my ( $output, $error ) = @_;
-   if ( defined($error) and $error =~ /^$options{frontend}:\ (.*)/x ) {
+   if ( defined($error) and $error =~ /^$options{frontend}:\ (.*)/xsm ) {
     $error = $1;
    }
-   $options{error_callback}->($error)
-     if ( defined($error) and defined( $options{error_callback} ) );
+   if ( defined($error) and defined( $options{error_callback} ) ) {
+    $options{error_callback}->($error);
+   }
    my $options = Gscan2pdf::Scanner::Options->new_from_data($output);
    $_self->{device_name} = Gscan2pdf::Scanner::Options->device;
-   $options{finished_callback}->($options)
-     if ( defined $options{finished_callback} );
+   if ( defined $options{finished_callback} ) {
+    $options{finished_callback}->($options);
+   }
   }
  );
  return;
@@ -107,7 +120,7 @@ sub find_scan_options {
 sub scan_pages {
  my ( $class, %options ) = @_;
 
- $options{prefix} = '' unless ( defined $options{prefix} );
+ if ( not defined( $options{prefix} ) ) { $options{prefix} = $EMPTY }
 
  if (
   defined( $options{frontend} )
@@ -127,8 +140,9 @@ sub scan_pages {
 
 sub _scanimage {
  my (%options) = @_;
- $options{frontend} = 'scanimage'
-   if ( not defined( $options{frontend} ) or $options{frontend} eq '' );
+ if ( not defined( $options{frontend} ) or $options{frontend} eq $EMPTY ) {
+  $options{frontend} = 'scanimage';
+ }
 
  my $cmd = _create_scanimage_cmd( \%options, TRUE );
 
@@ -146,23 +160,26 @@ sub _scanimage {
   err_callback     => sub {
    my ($line) = @_;
    given ($line) {
-    when (/^Progress:\ (\d*\.\d*)%/x) {
-     $options{running_callback}->( $1 / 100 )
-       if ( defined $options{running_callback} );
+    when (/^Progress:\ (\d*\.\d*)%/xsm) {
+     if ( defined $options{running_callback} ) {
+      $options{running_callback}->( $1 / $_100 );
+     }
     }
-    when (/^Scanning\ (-?\d*)\ pages/x) {
-     $options{running_callback}
-       ->( 0, sprintf( $d->get('Scanning %i pages...'), $1 ) )
-       if ( defined $options{running_callback} );
+    when (/^Scanning\ (-?\d*)\ pages/xsm) {
+     if ( defined $options{running_callback} ) {
+      $options{running_callback}
+        ->( 0, sprintf( $d->get('Scanning %i pages...'), $1 ) );
+     }
     }
-    when (/^Scanning\ page\ (\d*)/x) {
-     $options{running_callback}
-       ->( 0, sprintf( $d->get('Scanning page %i...'), $1 ) )
-       if ( defined $options{running_callback} );
+    when (/^Scanning\ page\ (\d*)/xsm) {
+     if ( defined $options{running_callback} ) {
+      $options{running_callback}
+        ->( 0, sprintf( $d->get('Scanning page %i...'), $1 ) );
+     }
     }
-    when (/^Scanned\ page\ (\d*)\.\ \(scanner\ status\ =\ (\d)\)/x) {
+    when (/^Scanned\ page\ (\d*)\.\ \(scanner\ status\ =\ (\d)\)/xsm) {
      my ( $id, $return ) = ( $1, $2 );
-     if ( $return == 5 ) {
+     if ( $return == $_SANE_STATUS_EOF ) {
       my $timer = Glib::Timeout->add(
        $_POLL_INTERVAL,
        sub {
@@ -170,9 +187,10 @@ sub _scanimage {
           defined( $options{dir} )
           ? File::Spec->catfile( $options{dir}, "out$id.pnm" )
           : "out$id.pnm";
-        return Glib::SOURCE_CONTINUE unless ( -e $path );
-        $options{new_page_callback}->($id)
-          if ( defined $options{new_page_callback} );
+        if ( not -e $path ) { return Glib::SOURCE_CONTINUE }
+        if ( defined $options{new_page_callback} ) {
+         $options{new_page_callback}->($id);
+        }
         $num_scans++;
         return Glib::SOURCE_REMOVE;
        }
@@ -180,46 +198,51 @@ sub _scanimage {
      }
     }
     when (
-     /Scanner\ warming\ up\ -\ waiting\ \d*\ seconds|wait\ for\ lamp\ warm-up/x ## no critic (ProhibitComplexRegexes)
+/Scanner\ warming\ up\ -\ waiting\ \d*\ seconds|wait\ for\ lamp\ warm-up/xsm ## no critic (ProhibitComplexRegexes)
       )
     {
-     $options{running_callback}->( 0, $d->get('Scanner warming up') )
-       if ( defined $options{running_callback} );
+     if ( defined $options{running_callback} ) {
+      $options{running_callback}->( 0, $d->get('Scanner warming up') );
+     }
     }
     when (
-     /^$options{frontend}:\ sane_start:\ Document\ feeder\ out\ of\ documents/x
+/^$options{frontend}:\ sane_start:\ Document\ feeder\ out\ of\ documents/xsm
       )
     {
-     $options{error_callback}->( $d->get('Document feeder out of documents') )
-       if ( defined( $options{error_callback} ) and $num_scans == 0 );
+     if ( defined( $options{error_callback} ) and $num_scans == 0 ) {
+      $options{error_callback}->( $d->get('Document feeder out of documents') );
+     }
     }
     when (
      $_self->{abort_scan} == TRUE
        and ( $line =~
-         /^$options{frontend}:\ sane_start:\ Error\ during\ device\ I\/O/x
-      or $line =~ /^$options{frontend}:\ received\ signal\ 2/x
-      or $line =~ /^$options{frontend}:\ trying\ to\ stop\ scanner/x )
+         /^$options{frontend}:\ sane_start:\ Error\ during\ device\ I\/O/xsm
+      or $line =~ /^$options{frontend}:\ received\ signal\ 2/xsm
+      or $line =~ /^$options{frontend}:\ trying\ to\ stop\ scanner/xsm )
       )
     {
      ;
     }
-    when (/^$options{frontend}:\ rounded/x) {
+    when (/^$options{frontend}:\ rounded/xsm) {
      $logger->info( substr( $line, 0, index( $line, "\n" ) + 1 ) );
     }
-    when (/^$options{frontend}:\ sane_(?:start|read):\ Device\ busy/x) {
-     $options{error_callback}->( $d->get('Device busy') )
-       if ( defined $options{error_callback} );
+    when (/^$options{frontend}:\ sane_(?:start|read):\ Device\ busy/xsm) {
+     if ( defined $options{error_callback} ) {
+      $options{error_callback}->( $d->get('Device busy') );
+     }
     }
     when (
-     /^$options{frontend}:\ sane_(?:start|read):\ Operation\ was\ cancelled/x)
+     /^$options{frontend}:\ sane_(?:start|read):\ Operation\ was\ cancelled/xsm)
     {
-     $options{error_callback}->( $d->get('Operation cancelled') )
-       if ( defined $options{error_callback} );
+     if ( defined $options{error_callback} ) {
+      $options{error_callback}->( $d->get('Operation cancelled') );
+     }
     }
     default {
-     $options{error_callback}->(
-      $d->get('Unknown message: ') . substr( $line, 0, index( $line, "\n" ) ) )
-       if ( defined $options{error_callback} );
+     if ( defined $options{error_callback} ) {
+      $options{error_callback}->( $d->get('Unknown message: ')
+         . substr( $line, 0, index( $line, "\n" ) ) );
+     }
     }
    }
   },
@@ -232,11 +255,11 @@ sub _scanimage {
 
 sub _create_scanimage_cmd {
  my ( $options, $scan ) = @_;
- my %options = %$options;
+ my %options = %{$options};
 
- $options{frontend} = 'scanimage' unless ( defined $options{frontend} );
+ if ( not defined( $options{frontend} ) ) { $options{frontend} = 'scanimage' }
 
- my $help = $scan ? '' : '--help';
+ my $help = $scan ? $EMPTY : '--help';
 
  # inverted commas needed for strange characters in device name
  my $device = "--device-name='$options{device}'";
@@ -244,23 +267,26 @@ sub _create_scanimage_cmd {
  # Add basic options
  my @options;
  for ( @{ $options{options} } ) {
-  my ( $key, $value ) = each(%$_);
-  if ( $key =~ /^(?:x|y|t|l)$/x ) {
+  my ( $key, $value ) = each( %{$_} );
+  if ( $key =~ /^(?:x|y|t|l)$/xsm ) {
    push @options, "-$key $value";
   }
   else {
    push @options, "--$key='$value'";
   }
  }
- unless ($help) {
+ if ( not $help ) {
   push @options, '--batch';
   push @options, '--progress';
-  push @options, "--batch-start=$options{start}"
-    if ( defined( $options{start} ) and $options{start} != 0 );
-  push @options, "--batch-count=$options{npages}"
-    if ( defined( $options{npages} ) and $options{npages} != 0 );
-  push @options, "--batch-increment=$options{step}"
-    if ( defined( $options{step} ) and $options{step} != 1 );
+  if ( defined( $options{start} ) and $options{start} != 0 ) {
+   push @options, "--batch-start=$options{start}";
+  }
+  if ( defined( $options{npages} ) and $options{npages} != 0 ) {
+   push @options, "--batch-count=$options{npages}";
+  }
+  if ( defined( $options{step} ) and $options{step} != 1 ) {
+   push @options, "--batch-increment=$options{step}";
+  }
  }
 
  # Create command
@@ -272,16 +298,16 @@ sub _create_scanimage_cmd {
 sub _scanadf {
  my (%options) = @_;
 
- $options{frontend} = 'scanadf' unless ( defined $options{frontend} );
+ if ( not defined( $options{frontend} ) ) { $options{frontend} = 'scanadf' }
 
  # inverted commas needed for strange characters in device name
  my $device = "--device-name='$options{device}'";
 
  # Add basic options
  my @options;
- @options = @{ $options{options} } if ( defined $options{options} );
+ if ( defined $options{options} ) { @options = @{ $options{options} } }
  push @options, "--start-count=1";
- push @options, "--end-count=$options{npages}" if ( $options{npages} != 0 );
+ if ( $options{npages} != 0 ) { push @options, "--end-count=$options{npages}" }
  push @options, '-o out%d.pnm';
 
  # Create command
@@ -303,15 +329,19 @@ sub _scanadf {
       }
       else {
        # Pulse
-       $options{running_callback}->(-1);
+       $options{running_callback}->();
       }
      }
-     elsif ( -e "out$id.pnm" and ( -s "out$id.pnm" ) > 50 ) {
+
+     # 50 is enough of the file for the header to be complete
+     elsif ( -e "out$id.pnm"
+      and ( -s "out$id.pnm" ) > 50 )    ## no critic (ProhibitMagicNumbers)
+     {
       $size = Gscan2pdf::NetPBM::file_size_from_header("out$id.pnm");
      }
      else {
       # Pulse
-      $options{running_callback}->(-1);
+      $options{running_callback}->();
      }
      return Glib::SOURCE_CONTINUE;
     }
@@ -328,22 +358,24 @@ sub _scanadf {
    my ($line) = @_;
    given ($line) {
     when (
-     /Scanner\ warming\ up\ -\ waiting\ \d*\ seconds|wait\ for\ lamp\ warm-up/x ## no critic (ProhibitComplexRegexes)
+/Scanner\ warming\ up\ -\ waiting\ \d*\ seconds|wait\ for\ lamp\ warm-up/xsm ## no critic (ProhibitComplexRegexes)
       )
     {
-     $options{running_callback}->( 0, $d->get('Scanner warming up') )
-       if ( defined $options{running_callback} );
+     if ( defined $options{running_callback} ) {
+      $options{running_callback}->( 0, $d->get('Scanner warming up') );
+     }
     }
-    when (/^Scanned\ document\ out(\d*)\.pnm/x) {
+    when (/^Scanned\ document\ out(\d*)\.pnm/xsm) {
      $id = $1;
 
      # Timer will run until callback returns false
      my $timer = Glib::Timeout->add(
       $_POLL_INTERVAL,
       sub {
-       return Glib::SOURCE_CONTINUE unless ( -e "out$id.pnm" );
-       $options{new_page_callback}->($id)
-         if ( defined $options{new_page_callback} );
+       if ( not -e "out$id.pnm" ) { return Glib::SOURCE_CONTINUE }
+       if ( defined $options{new_page_callback} ) {
+        $options{new_page_callback}->($id);
+       }
        return Glib::SOURCE_REMOVE;
       }
      );
@@ -353,31 +385,36 @@ sub _scanadf {
      undef $size;
 
     }
-    when (/^Scanned\ \d*\ pages/x) {
+    when (/^Scanned\ \d*\ pages/xsm) {
      ;
     }
-    when (/^$options{frontend}:\ rounded/x) {
+    when (/^$options{frontend}:\ rounded/xsm) {
      $logger->info( substr( $line, 0, index( $line, "\n" ) + 1 ) );
     }
-    when (/^$options{frontend}:\ sane_start:\ Device\ busy/x) {
-     $options{error_callback}->( $d->get('Device busy') )
-       if ( defined $options{error_callback} );
+    when (/^$options{frontend}:\ sane_start:\ Device\ busy/xsm) {
+     if ( defined $options{error_callback} ) {
+      $options{error_callback}->( $d->get('Device busy') );
+     }
      $running = FALSE;
     }
-    when (/^$options{frontend}:\ sane_read:\ Operation\ was\ cancelled/x) {
-     $options{error_callback}->( $d->get('Operation cancelled') )
-       if ( defined $options{error_callback} );
+    when (/^$options{frontend}:\ sane_read:\ Operation\ was\ cancelled/xsm) {
+     if ( defined $options{error_callback} ) {
+      $options{error_callback}->( $d->get('Operation cancelled') );
+     }
      $running = FALSE;
     }
     default {
-     $options{error_callback}->(
-      $d->get('Unknown message: ') . substr( $line, 0, index( $line, "\n" ) ) )
-       if ( defined $options{error_callback} );
+     if ( defined $options{error_callback} ) {
+      $options{error_callback}->( $d->get('Unknown message: ')
+         . substr( $line, 0, index( $line, "\n" ) ) );
+     }
     }
    }
   },
   finished_callback => sub {
-   $options{finished_callback}->() if ( defined $options{finished_callback} );
+   if ( defined $options{finished_callback} ) {
+    $options{finished_callback}->();
+   }
    $running = FALSE;
   }
  );
@@ -400,6 +437,7 @@ sub _watch_cmd {
 
  my $out_finished = FALSE;
  my $err_finished = FALSE;
+ my $error_flag   = FALSE;
  $logger->info( $options{cmd} );
 
  if ( defined $options{running_callback} ) {
@@ -415,7 +453,7 @@ sub _watch_cmd {
 
  # Make sure we are in temp directory
  my $cwd = getcwd;
- chdir $options{dir} if ( defined $options{dir} );
+ if ( defined $options{dir} ) { chdir $options{dir} }
 
  # Interface to scanimage
  my ( $write, $read );
@@ -426,26 +464,30 @@ sub _watch_cmd {
  # change back to original directory
  chdir $cwd;
 
- $options{started_callback}->() if ( defined $options{started_callback} );
+ if ( defined $options{started_callback} ) { $options{started_callback}->() }
  if ( $_self->{abort_scan} ) {
   local $SIG{INT} = 'IGNORE';
   $logger->info("Sending INT signal to PID $pid and its children");
   killfam 'INT', ($pid);
  }
- my ( $stdout, $stderr );
+ my ( $stdout, $stderr, $error_message );
 
  _add_watch(
   $read,
   sub {
    my ($line) = @_;
    $stdout .= $line;
-   $options{out_callback}->($line) if ( defined $options{out_callback} );
+   if ( defined $options{out_callback} ) { $options{out_callback}->($line) }
   },
   sub {
 
    # Don't flag this until after the callback to avoid the race condition
    # where stdout is truncated by stderr prematurely reaping the process
    $out_finished = TRUE;
+  },
+  sub {
+   ($error_message) = @_;
+   $error_flag = TRUE;
   }
  );
  _add_watch(
@@ -453,13 +495,17 @@ sub _watch_cmd {
   sub {
    my ($line) = @_;
    $stderr .= $line;
-   $options{err_callback}->($line) if ( defined $options{err_callback} );
+   if ( defined $options{err_callback} ) { $options{err_callback}->($line) }
   },
   sub {
 
    # Don't flag this until after the callback to avoid the race condition
    # where stderr is truncated by stdout prematurely reaping the process
    $err_finished = TRUE;
+  },
+  sub {
+   ($error_message) = @_;
+   $error_flag = TRUE;
   }
  );
 
@@ -473,14 +519,23 @@ sub _watch_cmd {
    my $timer = Glib::Timeout->add(
     $_POLL_INTERVAL,
     sub {
-     if ( $out_finished and $err_finished ) {
+     if ($error_flag) {
+      if ( defined $options{error_callback} ) {
+       $options{error_callback}->($error_message);
+      }
+      return Glib::SOURCE_REMOVE;
+     }
+     elsif ( $out_finished and $err_finished ) {
 
-      $options{finished_callback}->( $stdout, $stderr )
-        if ( defined $options{finished_callback} );
+      if ( defined $options{finished_callback} ) {
+       $options{finished_callback}->( $stdout, $stderr );
+      }
       $logger->info('Waiting to reap process');
 
-      # So we don't leave zombies
-      $logger->info( "Reaped PID ", waitpid( -1, &WNOHANG ) );
+      # -1 indicates a non-blocking wait for all pending zombie processes
+      $logger->info( "Reaped PID ",
+       waitpid( -1, WNOHANG ) )    ## no critic (ProhibitMagicNumbers)
+        ;
       return Glib::SOURCE_REMOVE;
      }
      return Glib::SOURCE_CONTINUE;
@@ -492,7 +547,7 @@ sub _watch_cmd {
 }
 
 sub _add_watch {
- my ( $fh, $line_callback, $finished_callback ) = @_;
+ my ( $fh, $line_callback, $finished_callback, $error_callback ) = @_;
  my $line;
  Glib::IO->add_watch(
   fileno($fh),
@@ -504,22 +559,30 @@ sub _add_watch {
 
     # Only reading one buffer, rather than until sysread gives EOF
     # because things seem to be strange for stderr
-    sysread $fh, $buffer, 1024;
-    $line .= $buffer if ($buffer);
+    sysread $fh, $buffer, $_1KB;
+    if ($buffer) { $line .= $buffer }
 
-    while ( $line =~ /([\r\n])/x ) {
+    while ( $line =~ /([\r\n])/xsm ) {
      my $le = $1;
-     $line_callback->( substr( $line, 0, index( $line, $le ) + 1 ) )
-       if ( defined $line_callback );
+     if ( defined $line_callback ) {
+      $line_callback->( substr( $line, 0, index( $line, $le ) + 1 ) );
+     }
      $line = substr( $line, index( $line, $le ) + 1, length($line) );
     }
    }
 
    # Only allow the hup if sure an empty buffer has been read.
-   if ( ( $condition & 'hup' ) and ( not defined($buffer) or $buffer eq '' ) )
-   {    # bit field operation. >= would also work
-    close $fh;
-    $finished_callback->();
+   if (
+    ( $condition & 'hup' )    # bit field operation. >= would also work
+    and ( not defined($buffer) or $buffer eq $EMPTY )
+     )
+   {
+    if ( close $fh ) {
+     $finished_callback->();
+    }
+    elsif ( defined $error_callback ) {
+     $error_callback->('Error closing filehandle');
+    }
     return Glib::SOURCE_REMOVE;
    }
    return Glib::SOURCE_CONTINUE;
