@@ -10,17 +10,23 @@ use Thread::Queue;
 
 use Glib qw(TRUE FALSE);
 use Sane;
+use Readonly;
+Readonly my $BUFFER_SIZE    => ( 32 * 1024 );     # default size
+Readonly my $_POLL_INTERVAL => 100;               # ms
+Readonly my $_8_BIT         => 8;
+Readonly my $MAXVAL_8_BIT   => 2**$_8_BIT - 1;
+Readonly my $_16_BIT        => 16;
+Readonly my $MAXVAL_16_BIT  => 2**$_16_BIT - 1;
+
+our $VERSION = '1.2.3';
 
-my $_POLL_INTERVAL;
 my $_self;
-my $buffer_size = ( 32 * 1024 );    # default size
 my ( $prog_name, $logger );
 
 sub setup {
  ( my $class, $logger ) = @_;
- $_POLL_INTERVAL = 100;                          # ms
- $_self          = {};
- $prog_name      = Glib::get_application_name;
+ $_self     = {};
+ $prog_name = Glib::get_application_name;
 
  $_self->{requests} = Thread::Queue->new;
  share $_self->{device_list};
@@ -53,7 +59,7 @@ sub _when_ready {
  Glib::Timeout->add(
   $_POLL_INTERVAL,
   sub {
-   if ($$sentinel) {
+   if ( ${$sentinel} ) {
     $ready_callback->();
     return Glib::SOURCE_REMOVE;
    }
@@ -84,11 +90,11 @@ sub get_devices {
  _when_ready(
   $sentinel,
   sub {
-   $started_callback->() unless ($started);
+   if ( not $started ) { $started_callback->() }
    $finished_callback->( $_self->{device_list} );
   },
   sub {
-   unless ($started) {
+   if ( not $started ) {
     $started_callback->();
     $started = 1;
    }
@@ -116,22 +122,26 @@ sub open_device {
  _when_ready(
   $sentinel,
   sub {
-   $options{started_callback}->()
-     if ( not $started and defined( $options{started_callback} ) );
+   if ( not $started and defined( $options{started_callback} ) ) {
+    $options{started_callback}->();
+   }
    if ( $_self->{status} == SANE_STATUS_GOOD ) {
-    $options{finished_callback}->() if ( defined $options{finished_callback} );
+    if ( defined $options{finished_callback} ) {
+     $options{finished_callback}->();
+    }
    }
    else {
-    $options{error_callback}->( Sane::strstatus( $_self->{status} ) )
-      if ( defined $options{error_callback} );
+    if ( defined $options{error_callback} ) {
+     $options{error_callback}->( Sane::strstatus( $_self->{status} ) );
+    }
    }
   },
   sub {
-   unless ($started) {
-    $options{started_callback}->() if ( defined $options{started_callback} );
+   if ( not $started ) {
+    if ( defined $options{started_callback} ) { $options{started_callback}->() }
     $started = 1;
    }
-   $options{running_callback}->() if ( defined $options{running_callback} );
+   if ( defined $options{running_callback} ) { $options{running_callback}->() }
   }
  );
  return;
@@ -151,7 +161,7 @@ sub find_scan_options {
  _when_ready(
   $sentinel,
   sub {
-   $started_callback->() unless ($started);
+   if ( not $started ) { $started_callback->() }
    if ( $_self->{status} == SANE_STATUS_GOOD ) {
     $finished_callback->($option_array);
    }
@@ -160,7 +170,7 @@ sub find_scan_options {
    }
   },
   sub {
-   unless ($started) {
+   if ( not $started ) {
     $started_callback->();
     $started = 1;
    }
@@ -187,17 +197,19 @@ sub set_option {
  _when_ready(
   $sentinel,
   sub {
-   $options{started_callback}->()
-     if ( not $started and defined( $options{started_callback} ) );
-   $options{finished_callback}->($option_array)
-     if ( defined $options{finished_callback} );
+   if ( not $started and defined( $options{started_callback} ) ) {
+    $options{started_callback}->();
+   }
+   if ( defined $options{finished_callback} ) {
+    $options{finished_callback}->($option_array);
+   }
   },
   sub {
-   unless ($started) {
-    $options{started_callback}->() if ( defined $options{started_callback} );
+   if ( not $started ) {
+    if ( defined $options{started_callback} ) { $options{started_callback}->() }
     $started = 1;
    }
-   $options{running_callback}->() if ( defined $options{running_callback} );
+   if ( defined $options{running_callback} ) { $options{running_callback}->() }
   }
  );
  return;
@@ -206,7 +218,7 @@ sub set_option {
 sub _new_page {
  my ( $dir, $format, $n ) = @_;
  my $path = sprintf $format, $n;
- $path = File::Spec->catdir( $dir, $path ) if ( defined $dir );
+ if ( defined $dir ) { $path = File::Spec->catdir( $dir, $path ) }
  return _enqueue_request( 'scan-page', { path => $path } );
 }
 
@@ -223,49 +235,55 @@ sub scan_pages {
  Glib::Timeout->add(
   $_POLL_INTERVAL,
   sub {
-   if ($$sentinel) {
+   if ( ${$sentinel} ) {
 
     # Check status of scan
-    $options{new_page_callback}->( $options{start} )
-      if (
+    if (
          defined( $options{new_page_callback} )
      and not $_self->{abort_scan}
      and ( $_self->{status} == SANE_STATUS_GOOD
       or $_self->{status} == SANE_STATUS_EOF )
-      );
+      )
+    {
+     $options{new_page_callback}->( $options{start} );
+    }
 
     # Stop the process unless everything OK and more scans required
     if (
         $_self->{abort_scan}
-     or ( $options{npages} != -1 and not --$options{npages} )
+     or ( $options{npages} and ++$n > $options{npages} )
      or ( $_self->{status} != SANE_STATUS_GOOD
       and $_self->{status} != SANE_STATUS_EOF )
       )
     {
      _enqueue_request('cancel');
      if ( _scanned_enough_pages( $options{npages}, $n ) ) {
-      $options{finished_callback}->()
-        if ( defined $options{finished_callback} );
+      if ( defined $options{finished_callback} ) {
+       $options{finished_callback}->();
+      }
      }
      else {
-      $options{error_callback}->( Sane::strstatus( $_self->{status} ) )
-        if ( defined $options{error_callback} );
+      if ( defined $options{error_callback} ) {
+       $options{error_callback}->( Sane::strstatus( $_self->{status} ) );
+      }
      }
      return Glib::SOURCE_REMOVE;
     }
 
     $options{start} += $options{step};
-    $n++;
     $sentinel = _new_page( $options{dir}, $options{format}, $options{start} );
     return Glib::SOURCE_CONTINUE;
    }
    else {
-    unless ($started) {
-     $options{started_callback}->() if ( defined $options{started_callback} );
+    if ( not $started ) {
+     if ( defined $options{started_callback} ) {
+      $options{started_callback}->();
+     }
      $started = 1;
     }
-    $options{running_callback}->( $_self->{scan_progress} )
-      if ( defined $options{running_callback} );
+    if ( defined $options{running_callback} ) {
+     $options{running_callback}->( $_self->{scan_progress} );
+    }
     return Glib::SOURCE_CONTINUE;
    }
   }
@@ -274,13 +292,12 @@ sub scan_pages {
 }
 
 sub _scanned_enough_pages {
- my ( $ntodo, $ndone ) = @_;
+ my ( $nrequired, $ndone ) = @_;
  return (
        $_self->{status} == SANE_STATUS_GOOD
     or $_self->{status} == SANE_STATUS_EOF
     or ( $_self->{status} == SANE_STATUS_NO_DOCS
-   and $ntodo < 1
-   and $ndone > 1 )
+   and $nrequired < $ndone )
  );
 }
 
@@ -340,7 +357,7 @@ sub _thread_open_device {
  my ( $self, $device_name ) = @_;
 
  # close the handle
- undef $self->{device_handle} if ( defined( $self->{device_handle} ) );
+ if ( defined( $self->{device_handle} ) ) { undef $self->{device_handle} }
 
  $self->{device_handle} = Sane::Device->open($device_name);
  $logger->debug("opening device '$device_name': $Sane::STATUS");
@@ -362,18 +379,20 @@ sub _thread_get_options {
   $logger->error("unable to determine option count");
   return;
  }
- for ( my $i = 1 ; $i < $num_dev_options ; ++$i ) {
-  my $opt = $self->{device_handle}->get_option_descriptor($i);
-  $options[$i] = $opt;
-  $opt->{val} = $self->{device_handle}->get_option($i)
-    if (
+ for ( 1 .. $num_dev_options - 1 ) {
+  my $opt = $self->{device_handle}->get_option_descriptor($_);
+  $options[$_] = $opt;
+  if (
    not(( $opt->{cap} & SANE_CAP_INACTIVE )
     or ( $opt->{type} == SANE_TYPE_BUTTON )
     or ( $opt->{type} == SANE_TYPE_GROUP ) )
-    );
+    )
+  {
+   $opt->{val} = $self->{device_handle}->get_option($_);
+  }
  }
 
- $$options = shared_clone \@options;
+ ${$options} = shared_clone \@options;
  return;
 }
 
@@ -402,16 +421,17 @@ sub _thread_set_option {
   }
 
   my @options;
-  for ( my $i = 1 ; $i < $num_dev_options ; ++$i ) {
-   my $opt = $self->{device_handle}->get_option_descriptor($i);
-   $options[$i] = $opt;
-   next if ( !( $opt->{cap} & SANE_CAP_SOFT_DETECT ) );
+  for ( 1 .. $num_dev_options - 1 ) {
+   my $opt = $self->{device_handle}->get_option_descriptor($_);
+   $options[$_] = $opt;
+   if ( not $opt->{cap} & SANE_CAP_SOFT_DETECT ) { next }
 
-   $opt->{val} = $self->{device_handle}->get_option($i)
-     if ( $opt->{type} != SANE_TYPE_BUTTON );
+   if ( $opt->{type} != SANE_TYPE_BUTTON ) {
+    $opt->{val} = $self->{device_handle}->get_option($_);
+   }
   }
 
-  $$new_options = shared_clone \@options;
+  ${$new_options} = shared_clone \@options;
  }
  return;
 }
@@ -429,7 +449,7 @@ sub _thread_write_pnm_header {
   or $format == SANE_FRAME_RGB )
  {
   printf $fh "P6\n# SANE data follows\n%d %d\n%d\n", $width, $height,
-    ( $depth <= 8 ) ? 255 : 65535;
+    ( $depth > $_8_BIT ) ? $MAXVAL_16_BIT : $MAXVAL_8_BIT;
  }
  else {
   if ( $depth == 1 ) {
@@ -437,7 +457,7 @@ sub _thread_write_pnm_header {
   }
   else {
    printf $fh "P5\n# SANE data follows\n%d %d\n%d\n", $width, $height,
-     ( $depth <= 8 ) ? 255 : 65535;
+     ( $depth > $_8_BIT ) ? $MAXVAL_16_BIT : $MAXVAL_8_BIT;
   }
  }
  return;
@@ -448,79 +468,74 @@ sub _thread_scan_page_to_fh {
  my $first_frame = 1;
  my $offset      = 0;
  my $must_buffer = 0;
- my $min         = 0xff;
- my $max         = 0;
  my %image;
  my @format_name = ( "gray", "RGB", "red", "green", "blue" );
  my $total_bytes = 0;
 
- my $parm;
- {
-  do {    # extra braces to get last to work.
-   if ( not $first_frame ) {
-    $device->start;
-    if ( $Sane::STATUS != SANE_STATUS_GOOD ) {
-     $logger->info("$prog_name: sane_start: $Sane::STATUS");
-     goto cleanup;
-    }
-   }
-
-   $parm = $device->get_parameters;
+ my ( $parm, $last_frame );
+ while ( not $last_frame ) {
+  if ( not $first_frame ) {
+   $device->start;
    if ( $Sane::STATUS != SANE_STATUS_GOOD ) {
-    $logger->info("$prog_name: sane_get_parameters: $Sane::STATUS");
+    $logger->info("$prog_name: sane_start: $Sane::STATUS");
     goto cleanup;
    }
+  }
 
-   _log_frame_info( $first_frame, $parm, \@format_name );
-   ( $must_buffer, $offset ) = _initialise_scan( $fh, $first_frame, $parm );
-   my $hundred_percent = _scan_data_size($parm);
+  $parm = $device->get_parameters;
+  if ( $Sane::STATUS != SANE_STATUS_GOOD ) {
+   $logger->info("$prog_name: sane_get_parameters: $Sane::STATUS");
+   goto cleanup;
+  }
 
-   while (1) {
+  _log_frame_info( $first_frame, $parm, \@format_name );
+  ( $must_buffer, $offset ) = _initialise_scan( $fh, $first_frame, $parm );
+  my $hundred_percent = _scan_data_size($parm);
 
-    # Pick up flag from cancel_scan()
-    if ( $_self->{abort_scan} ) {
-     $device->cancel;
-     $logger->info('Scan cancelled');
+  while (1) {
+
+   # Pick up flag from cancel_scan()
+   if ( $_self->{abort_scan} ) {
+    $device->cancel;
+    $logger->info('Scan cancelled');
+    return;
+   }
+
+   my ( $buffer, $len ) = $device->read($BUFFER_SIZE);
+   $total_bytes += $len;
+   my $progr = $total_bytes / $hundred_percent;
+   if ( $progr > 1 ) { $progr = 1 }
+   $_self->{scan_progress} = $progr;
+
+   if ( $Sane::STATUS != SANE_STATUS_GOOD ) {
+    if ( $parm->{depth} == $_8_BIT ) {
+     $logger->info( sprintf "$prog_name: min/max graylevel value = %d/%d",
+      $MAXVAL_8_BIT, 0 );
+    }
+    if ( $Sane::STATUS != SANE_STATUS_EOF ) {
+     $logger->info("$prog_name: sane_read: $Sane::STATUS");
      return;
     }
-
-    my ( $buffer, $len ) = $device->read($buffer_size);
-    $total_bytes += $len;
-    my $progr = $total_bytes / $hundred_percent;
-    $progr = 1 if ( $progr > 1 );
-    $_self->{scan_progress} = $progr;
-
-    if ( $Sane::STATUS != SANE_STATUS_GOOD ) {
-     $logger->info( sprintf "$prog_name: min/max graylevel value = %d/%d",
-      $min, $max )
-       if ( $parm->{depth} == 8 );
-     if ( $Sane::STATUS != SANE_STATUS_EOF ) {
-      $logger->info("$prog_name: sane_read: $Sane::STATUS");
-      return;
-     }
-     last;
-    }
-
-    if ($must_buffer) {
-     $offset = _buffer_scan( $offset, $parm, \%image, $len, $buffer );
-    }
-    else {
-     print $fh $buffer;
-    }
+    last;
    }
-   $first_frame = 0;
-  } while ( !$parm->{last_frame} );
+
+   if ($must_buffer) {
+    $offset = _buffer_scan( $offset, $parm, \%image, $len, $buffer );
+   }
+   else {
+    print $fh $buffer;
+   }
+  }
+  $first_frame = 0;
+  $last_frame  = $parm->{last_frame};
  }
 
- _write_buffer_to_fh( $fh, $parm, \%image ) if ($must_buffer);
+ if ($must_buffer) { _write_buffer_to_fh( $fh, $parm, \%image ) }
 
 cleanup:
- my $expected_bytes = $parm->{bytes_per_line} * $parm->{lines} * (
-  ( $parm->{format} == SANE_FRAME_RGB or $parm->{format} == SANE_FRAME_GRAY )
-  ? 1
-  : 3
- );
- $expected_bytes = 0 if ( $parm->{lines} < 0 );
+ my $expected_bytes =
+   $parm->{bytes_per_line} * $parm->{lines} * _number_frames($parm);
+ if ( $parm->{lines} < 0 ) { $expected_bytes = 0 }
  if ( $total_bytes > $expected_bytes and $expected_bytes != 0 ) {
   $logger->info(
    sprintf "%s: WARNING: read more data than announced by backend " . "(%u/%u)",
@@ -536,7 +551,7 @@ cleanup:
 sub _thread_scan_page {
  my ( $self, $path ) = @_;
 
- unless ( defined $self->{device_handle} ) {
+ if ( not defined( $self->{device_handle} ) ) {
   $logger->info("$prog_name: must open device before starting scan");
   return;
  }
@@ -549,7 +564,7 @@ sub _thread_scan_page {
  }
 
  my $fh;
- if ( not open( $fh, ">", $path ) ) {    ## no critic (RequireBriefOpen)
+ if ( not open( $fh, ">", $path ) ) {
   $self->{device_handle}->cancel;
   $self->{status} = SANE_STATUS_ACCESS_DENIED;
   return;
@@ -558,7 +573,11 @@ sub _thread_scan_page {
  _thread_scan_page_to_fh( $self->{device_handle}, $fh );
  $self->{status} = $Sane::STATUS + 0;
 
- close $fh;
+ if ( not close($fh) ) {
+  $self->{device_handle}->cancel;
+  $self->{status} = SANE_STATUS_ACCESS_DENIED;
+  return;
+ }
 
  $logger->info( sprintf "Scanned page %s. (scanner status = %d)",
   $path, $Sane::STATUS );
@@ -574,7 +593,7 @@ sub _thread_scan_page {
 
 sub _thread_cancel {
  my ($self) = @_;
- $self->{device_handle}->cancel if ( defined $self->{device_handle} );
+ if ( defined $self->{device_handle} ) { $self->{device_handle}->cancel }
  return;
 }
 
@@ -587,7 +606,7 @@ sub _log_frame_info {
       . "%d bits/pixel",
     $parm->{pixels_per_line},
     $parm->{lines},
-    8 * $parm->{bytes_per_line} / $parm->{pixels_per_line}
+    $_8_BIT * $parm->{bytes_per_line} / $parm->{pixels_per_line}
    );
   }
   else {
@@ -595,7 +614,7 @@ sub _log_frame_info {
     sprintf "$prog_name: scanning image %d pixels wide and "
       . "variable height at %d bits/pixel",
     $parm->{pixels_per_line},
-    8 * $parm->{bytes_per_line} / $parm->{pixels_per_line}
+    $_8_BIT * $parm->{bytes_per_line} / $parm->{pixels_per_line}
    );
   }
 
@@ -617,22 +636,26 @@ sub _initialise_scan {
    or $parm->{format} == SANE_FRAME_GREEN
    or $parm->{format} == SANE_FRAME_BLUE )
   {
-   die "Red/Green/Blue frames require depth=8\n"
-     unless ( $parm->{depth} == 8 );
+   if ( $parm->{depth} != $_8_BIT ) {
+    die "Red/Green/Blue frames require depth=$_8_BIT\n";
+   }
    $must_buffer = 1;
    $offset      = $parm->{format} - SANE_FRAME_RED;
   }
   elsif ( $parm->{format} == SANE_FRAME_RGB ) {
-   die "RGB frames require depth=8 or 16\n"
-     unless ( ( $parm->{depth} == 8 ) or ( $parm->{depth} == 16 ) );
+   if ( ( $parm->{depth} != $_8_BIT ) and ( $parm->{depth} != $_16_BIT ) ) {
+    die "RGB frames require depth=$_8_BIT or $_16_BIT\n";
+   }
   }
   if ($parm->{format} == SANE_FRAME_RGB
    or $parm->{format} == SANE_FRAME_GRAY )
   {
-   die "Valid depths are 1, 8 or 16\n"
-     unless ( ( $parm->{depth} == 1 )
-    or ( $parm->{depth} == 8 )
-    or ( $parm->{depth} == 16 ) );
+   if ( ( $parm->{depth} != 1 )
+    and ( $parm->{depth} != $_8_BIT )
+    and ( $parm->{depth} != $_16_BIT ) )
+   {
+    die "Valid depths are 1, $_8_BIT or $_16_BIT\n";
+   }
    if ( $parm->{lines} < 0 ) {
     $must_buffer = 1;
     $offset      = 0;
@@ -656,11 +679,19 @@ sub _initialise_scan {
 
 sub _scan_data_size {
  my ($parm) = @_;
- return $parm->{bytes_per_line} * $parm->{lines} * (
-  ( $parm->{format} == SANE_FRAME_RGB or $parm->{format} == SANE_FRAME_GRAY )
-  ? 1
-  : 3
- );
+ return $parm->{bytes_per_line} * $parm->{lines} * _number_frames($parm);
+}
+
+# Return number of frames
+
+sub _number_frames {
+ my ($parm) = @_;
+ return (
+       $parm->{format} == SANE_FRAME_RGB
+    or $parm->{format} == SANE_FRAME_GRAY
+   )
+   ? 1
+   : 3;    ## no critic (ProhibitMagicNumbers)
 }
 
 # We're either scanning a multi-frame image or the
@@ -672,14 +703,9 @@ sub _scan_data_size {
 sub _buffer_scan {
  my ( $offset, $parm, $image, $len, $buffer ) = @_;
 
- # $parm->{format} == SANE_FRAME_RED or SANE_FRAME_GREEN or SANE_FRAME_BLUE
- my $number_frames = 3;
- $number_frames = 1
-   if ( $parm->{format} == SANE_FRAME_RGB
-  or $parm->{format} == SANE_FRAME_GRAY );
-
- for ( my $i = 0 ; $i < $len ; ++$i ) {
-  $image->{data}[ $offset + $number_frames * $i ] = substr( $buffer, $i, 1 );
+ my $number_frames = _number_frames($parm);
+ for ( 0 .. $len - 1 ) {
+  $image->{data}[ $offset + $number_frames * $_ ] = substr( $buffer, $_, 1 );
  }
  $offset += $number_frames * $len;
  return $offset;
@@ -692,10 +718,7 @@ sub _write_buffer_to_fh {
  }
  else {
   $image->{height} = @{ $image->{data} } / $parm->{pixels_per_line};
-  $image->{height} /= 3
-    if ( $parm->{format} == SANE_FRAME_RED
-   or $parm->{format} == SANE_FRAME_GREEN
-   or $parm->{format} == SANE_FRAME_BLUE );
+  $image->{height} /= _number_frames($parm);
  }
  _thread_write_pnm_header( $fh, $parm->{format}, $parm->{pixels_per_line},
   $image->{height}, $parm->{depth} );
