@@ -11,15 +11,23 @@ use HTML::TokeParser;
 use HTML::Entities;
 use Image::Magick;
 use Encode;
-use Locale::gettext 1.05;    # For translations
+use Locale::gettext 1.05;            # For translations
 use POSIX qw(locale_h);
+use English qw( -no_match_vars );    # for $ERRNO
+use Readonly;
+Readonly my $CM_PER_INCH    => 2.54;
+Readonly my $MM_PER_CM      => 10;
+Readonly my $MM_PER_INCH    => $CM_PER_INCH * $MM_PER_CM;
+Readonly my $PAGE_TOLERANCE => 0.02;
 
 BEGIN {
  use Exporter ();
  our ( $VERSION, @EXPORT_OK, %EXPORT_TAGS );
 
+ $VERSION = '1.2.3';
+
  use base qw(Exporter);
- %EXPORT_TAGS = ();          # eg: TAG => [ qw!name1 name2! ],
+ %EXPORT_TAGS = ();    # eg: TAG => [ qw!name1 name2! ],
 
  # your exported package globals go here,
  # as well as any optionally exported functions
@@ -32,10 +40,15 @@ my ( $d, $logger );
 sub new {
  my ( $class, %options ) = @_;
  my $self = {};
- $d = Locale::gettext->domain(Glib::get_application_name) unless ( defined $d );
+ if ( not defined($d) ) {
+  $d = Locale::gettext->domain(Glib::get_application_name);
+ }
 
- croak "Error: filename not supplied" unless ( defined $options{filename} );
- croak "Error: format not supplied"   unless ( defined $options{format} );
+ if ( not defined( $options{filename} ) ) {
+  croak "Error: filename not supplied";
+ }
+ if ( not -f $options{filename} )       { croak "Error: filename not found" }
+ if ( not defined( $options{format} ) ) { croak "Error: format not supplied" }
 
  $logger->info("New page filename $options{filename}, format $options{format}");
  for ( keys %options ) {
@@ -60,13 +73,13 @@ sub new {
  );
  if ( defined( $options{delete} ) and $options{delete} ) {
   move( $options{filename}, $self->{filename} )
-    or croak
-    sprintf( $d->get('Error importing image %s: %s'), $options{filename}, $! );
+    or croak sprintf( $d->get('Error importing image %s: %s'),
+   $options{filename}, $ERRNO );
  }
  else {
   copy( $options{filename}, $self->{filename} )
-    or croak
-    sprintf( $d->get('Error importing image %s: %s'), $options{filename}, $! );
+    or croak sprintf( $d->get('Error importing image %s: %s'),
+   $options{filename}, $ERRNO );
  }
 
  bless( $self, $class );
@@ -93,10 +106,12 @@ sub clone {
 sub freeze {
  my ($self) = @_;
  my $new = $self->clone;
- $new->{filename} = $self->{filename}->filename
-   if ( ref( $new->{filename} ) eq 'File::Temp' );
- $new->{dir} = $self->{dir}->dirname
-   if ( ref( $new->{dir} ) eq 'File::Temp::Dir' );
+ if ( ref( $new->{filename} ) eq 'File::Temp' ) {
+  $new->{filename} = $self->{filename}->filename;
+ }
+ if ( ref( $new->{dir} ) eq 'File::Temp::Dir' ) {
+  $new->{dir} = $self->{dir}->dirname;
+ }
  return $new;
 }
 
@@ -104,7 +119,7 @@ sub thaw {
  my ($self) = @_;
  my $new = $self->clone;
  my $suffix;
- if ( $new->{filename} =~ /\.(\w*)$/x ) {
+ if ( $new->{filename} =~ /\.(\w*)$/xsm ) {
   $suffix = $1;
  }
  my $filename = File::Temp->new( DIR => $new->{dir}, SUFFIX => ".$suffix" );
@@ -125,7 +140,7 @@ sub boxes {
    decode_utf8(
   encode_utf8( HTML::Entities::decode_entities( $self->{hocr} ) ) );
 
- if ( $self->{hocr} =~ /<body>([\s\S]*)<\/body>/x ) {
+ if ( $self->{hocr} =~ /<body>([\s\S]*)<\/body>/xsm ) {
   my $p = HTML::TokeParser->new( \$self->{hocr} );
   my ( $x1, $y1, $x2, $y2, $text );
   while ( my $token = $p->get_token ) {
@@ -137,7 +152,7 @@ sub boxes {
       or $token->[2]{class} eq 'ocr_word'
       or $token->[2]{class} eq 'ocrx_word' )
      and defined( $token->[2]{title} )
-     and $token->[2]{title} =~ /bbox\ (\d+)\ (\d+)\ (\d+)\ (\d+)/x
+     and $token->[2]{title} =~ /bbox\ (\d+)\ (\d+)\ (\d+)\ (\d+)/xsm
       )
     {
      ( $x1, $y1, $x2, $y2 ) = ( $1, $2, $3, $4 );
@@ -150,7 +165,7 @@ sub boxes {
      undef $text;
     }
    }
-   if ( $token->[0] eq 'T' and $token->[1] !~ /^\s*$/x ) {
+   if ( $token->[0] eq 'T' and $token->[1] !~ /^\s*$/xsm ) {
     $text = $token->[1];
     chomp($text);
    }
@@ -158,8 +173,9 @@ sub boxes {
     undef $x1;
     undef $text;
    }
-   push @boxes, [ $x1, $y1, $x2, $y2, $text ]
-     if ( defined($x1) and defined($text) );
+   if ( defined($x1) and defined($text) ) {
+    push @boxes, [ $x1, $y1, $x2, $y2, $text ];
+   }
   }
  }
  else {
@@ -198,15 +214,17 @@ sub resolution {
  # Imagemagick always reports PNMs as 72ppi
  # Some versions of imagemagick report colour PNM as Portable pixmap (PPM)
  # B&W are Portable anymap
- if ( $format !~ /^Portable\ ...map/x ) {
+ if ( $format !~ /^Portable\ ...map/xsm ) {
   $self->{resolution} = $image->Get('x-resolution');
 
-  $self->{resolution} = $image->Get('y-resolution')
-    unless ( defined $self->{resolution} );
+  if ( not defined( $self->{resolution} ) ) {
+   $self->{resolution} = $image->Get('y-resolution');
+  }
 
   if ( $self->{resolution} ) {
-   $self->{resolution} *= 2.54
-     unless ( $image->Get('units') eq 'PixelsPerInch' );
+   if ( not $image->Get('units') eq 'PixelsPerInch' ) {
+    $self->{resolution} *= $CM_PER_INCH;
+   }
    return $self->{resolution};
   }
  }
@@ -227,21 +245,22 @@ sub resolution {
 
 sub matching_paper_sizes {
  my ( $self, $paper_sizes ) = @_;
- unless ( defined( $self->{height} ) and defined( $self->{width} ) ) {
+ if ( not( defined( $self->{height} ) and defined( $self->{width} ) ) ) {
   my $image = $self->im_object;
   $self->{width}  = $image->Get('width');
   $self->{height} = $image->Get('height');
  }
  my $ratio = $self->{height} / $self->{width};
- $ratio = 1 / $ratio if ( $ratio < 1 );
+ if ( $ratio < 1 ) { $ratio = 1 / $ratio }
  my %matching;
- for ( keys %$paper_sizes ) {
+ for ( keys %{$paper_sizes} ) {
   if ( $paper_sizes->{$_}{x} > 0
-   and abs( $ratio - $paper_sizes->{$_}{y} / $paper_sizes->{$_}{x} ) < 0.02 )
+   and abs( $ratio - $paper_sizes->{$_}{y} / $paper_sizes->{$_}{x} ) <
+   $PAGE_TOLERANCE )
   {
    $matching{$_} =
      ( ( $self->{height} > $self->{width} ) ? $self->{height} : $self->{width} )
-     / $paper_sizes->{$_}{y} * 25.4;
+     / $paper_sizes->{$_}{y} * $MM_PER_INCH;
   }
  }
  return \%matching;
@@ -253,7 +272,7 @@ sub im_object {
  my ($self) = @_;
  my $image  = Image::Magick->new;
  my $x      = $image->Read( $self->{filename} );
- $logger->warn($x) if "$x";
+ if ("$x") { $logger->warn($x) }
  return $image;
 }
 
