@@ -6,16 +6,22 @@ use Carp;
 use Glib qw(TRUE FALSE);    # To get TRUE and FALSE
 use Sane 0.05;              # For enums
 use feature "switch";
+use Readonly;
+Readonly my $MAX_VALUES => 255;
 
 use Glib::Object::Subclass Glib::Object::;
 
-my $units = qr/(pel|bit|mm|dpi|%|us)/x;
+our $VERSION = '1.2.3';
+
+my $units = qr{(pel|bit|mm|dpi|%|us)}xsm;
+my $EMPTY = q{};
+my $list  = ',...';
 my $device;
 
 sub new_from_data {
  my ( $class, $options ) = @_;
  my $self = $class->new();
- croak "Error: no options supplied" unless ( defined $options );
+ if ( not defined($options) ) { croak "Error: no options supplied" }
  if ( ref($options) eq 'ARRAY' ) {
 
   # do a two level clone to allow us to add extra keys to the option hashes
@@ -23,11 +29,12 @@ sub new_from_data {
   $self->{array} = \@options;
   for my $i ( 0 .. $#{$options} ) {
    my %option;
-   %option = %{ $options->[$i] } if ( defined $options->[$i] );
+   if ( defined $options->[$i] ) { %option = %{ $options->[$i] } }
    $option{index} = $i;
    push @options, \%option;
-   $self->{hash}{ $options[$i]{name} } = $options[$i]
-     if ( defined( $options[$i]{name} ) and $options[$i]{name} ne '' );
+   if ( defined( $options[$i]{name} ) and $options[$i]{name} ne $EMPTY ) {
+    $self->{hash}{ $options[$i]{name} } = $options[$i];
+   }
   }
  }
  else {
@@ -63,8 +70,9 @@ sub num_options {
 
 sub delete_by_index {
  my ( $self, $i ) = @_;
- delete $self->{hash}{ $self->{array}[$i]{name} }
-   if ( defined $self->{array}[$i]{name} );
+ if ( defined $self->{array}[$i]{name} ) {
+  delete $self->{hash}{ $self->{array}[$i]{name} };
+ }
  undef $self->{array}[$i];
  return;
 }
@@ -100,43 +108,88 @@ sub parse_geometry {
  if ( defined $self->{hash}{ scalar(SANE_NAME_SCAN_TL_X) } ) {
   $self->{geometry}{l} =
     $self->{hash}{ scalar(SANE_NAME_SCAN_TL_X) }{constraint}{min};
-  $self->{geometry}{x} =
-    $self->{hash}{ scalar(SANE_NAME_SCAN_BR_X) }{constraint}{max} -
-    $self->{geometry}{l}
-    if ( defined $self->{hash}{ scalar(SANE_NAME_SCAN_BR_X) } );
+  if ( defined $self->{hash}{ scalar(SANE_NAME_SCAN_BR_X) } ) {
+   $self->{geometry}{x} =
+     $self->{hash}{ scalar(SANE_NAME_SCAN_BR_X) }{constraint}{max} -
+     $self->{geometry}{l};
+  }
  }
  elsif ( defined $self->{hash}{l} ) {
   $self->{geometry}{l} = $self->{hash}{l}{constraint}{min};
-  $self->{geometry}{x} = $self->{hash}{x}{constraint}{max}
-    if ( defined $self->{hash}{x}{constraint}{max} );
+  if ( defined $self->{hash}{x}{constraint}{max} ) {
+   $self->{geometry}{x} = $self->{hash}{x}{constraint}{max};
+  }
  }
  if ( defined $self->{hash}{ scalar(SANE_NAME_SCAN_TL_Y) } ) {
   $self->{geometry}{t} =
     $self->{hash}{ scalar(SANE_NAME_SCAN_TL_Y) }{constraint}{min};
-  $self->{geometry}{y} =
-    $self->{hash}{ scalar(SANE_NAME_SCAN_BR_Y) }{constraint}{max} -
-    $self->{geometry}{t}
-    if ( defined $self->{hash}{ scalar(SANE_NAME_SCAN_BR_Y) } );
+  if ( defined $self->{hash}{ scalar(SANE_NAME_SCAN_BR_Y) } ) {
+   $self->{geometry}{y} =
+     $self->{hash}{ scalar(SANE_NAME_SCAN_BR_Y) }{constraint}{max} -
+     $self->{geometry}{t};
+  }
  }
  elsif ( defined $self->{hash}{t} ) {
   $self->{geometry}{t} = $self->{hash}{t}{constraint}{min};
-  $self->{geometry}{y} = $self->{hash}{y}{constraint}{max}
-    if ( defined $self->{hash}{y}{constraint}{max} );
+  if ( defined $self->{hash}{y}{constraint}{max} ) {
+   $self->{geometry}{y} = $self->{hash}{y}{constraint}{max};
+  }
  }
  return;
 }
 
+sub synonyms {
+ my ($name) = @_;
+ my @synonyms = (
+  [ scalar(SANE_NAME_PAGE_HEIGHT), 'pageheight' ],
+  [ scalar(SANE_NAME_PAGE_WIDTH),  'pagewidth' ],
+  [ scalar(SANE_NAME_SCAN_TL_X),   'l' ],
+  [ scalar(SANE_NAME_SCAN_TL_Y),   't' ],
+  [ scalar(SANE_NAME_SCAN_BR_X),   'x' ],
+  [ scalar(SANE_NAME_SCAN_BR_Y),   'y' ]
+ );
+ for my $synonym (@synonyms) {
+  given ($name) {
+   when ( @{$synonym} ) {
+    return $synonym;
+   }
+  }
+ }
+ return [$name];
+}
+
+# Note any duplicate options, keeping only the last entry.
+sub prune_duplicates {
+ my ($arrayref) = @_;
+ my %seen;
+
+ my $j = $#{$arrayref};
+ while ( $j > -1 ) {    ## no critic (ProhibitMagicNumbers)
+  my ($opt) = keys( %{ $arrayref->[$j] } );
+  $seen{$opt}++;
+  my $synonyms = synonyms($opt);
+  for ( @{$synonyms} ) {
+   if ( defined( $seen{$_} ) and $seen{$_} > 1 ) {
+    splice @{$arrayref}, $j, 1;
+    last;
+   }
+  }
+  $j--;
+ }
+ return $arrayref;
+}
+
 sub supports_paper {
  my ( $self, $paper, $tolerance ) = @_;
 
  # Check the geometry against the paper size
- unless (   ## no critic (ProhibitNegativeExpressionsInUnlessAndUntilConditions)
-      defined( $self->{geometry}{l} )
-  and defined( $self->{geometry}{x} )
-  and defined( $self->{geometry}{t} )
-  and defined( $self->{geometry}{y} )
-  and $self->{geometry}{l} <= $paper->{l} + $tolerance
-  and $self->{geometry}{t} <= $paper->{t} + $tolerance
+ if (
+  not( defined( $self->{geometry}{l} )
+   and defined( $self->{geometry}{x} )
+   and defined( $self->{geometry}{t} )
+   and defined( $self->{geometry}{y} )
+   and $self->{geometry}{l} <= $paper->{l} + $tolerance
+   and $self->{geometry}{t} <= $paper->{t} + $tolerance )
    )
  {
   return 0;
@@ -169,10 +222,10 @@ sub options2hash {
  # Remove everything above the options
  if (
   $output =~ /
-                       [\S\s]* # output header
+                       .* # output header
                        Options\ specific\ to\ device\ `(.+)':\n # line above options
-                       ([\S\s]*) # options
-                /x
+                       (.*) # options
+                /xsm
    )
  {
   $device = $1;
@@ -188,24 +241,24 @@ sub options2hash {
   my %option;
   $option{unit}            = SANE_UNIT_NONE;
   $option{constraint_type} = SANE_CONSTRAINT_NONE;
-  my $values = qr/(?:(?:\ |\[=\()([^\[].*?)(?:\)\])?)?/x;
+  my $values = qr{(?:(?:\ |\[=\()([^\[].*?)(?:\)\])?)?}xsm;
 
   # parse group
   if (
    $output =~ /
-                      ^\ \ # two-character indent
-                      (.*) # the title
+                      \A\ \ # two-character indent
+                      ([^\n]*) # the title
                       :\n  # a colon at the end of the line
-                      ([\S\s]*) # the rest of the output
-                    /x
+                      (.*) # the rest of the output
+                    /xsm
     )
   {
    $option{title}      = $1;
    $option{type}       = SANE_TYPE_GROUP;
    $option{cap}        = 0;
    $option{max_values} = 0;
-   $option{name}       = '';
-   $option{desc}       = '';
+   $option{name}       = $EMPTY;
+   $option{desc}       = $EMPTY;
 
    # Remove everything on the option line and above.
    $output = $2;
@@ -214,14 +267,14 @@ sub options2hash {
   # parse option
   elsif (
    $output =~ /
-                      ^\ {4,} # four-character indent
+                      \A\ {4,} # four-character indent
                       -+        # at least one dash
                       ([\w\-]+) # the option name
                       $values      # optionally followed by the possible values
-                      (?:\ \[([\S\s]*?)\])?  # optionally a space, followed by the current value in square brackets
+                      (?:\ \[(.*?)\])?  # optionally a space, followed by the current value in square brackets
                       \ *\n     # the rest of the line
-                      ([\S\s]*) # the rest of the output
-                    /x
+                      (.*) # the rest of the output
+                    /xsm
     )
   {
 
@@ -253,10 +306,11 @@ sub options2hash {
    $hash{ $option{name} } = \%option;
 
    $option{title} = $option{name};
-   $option{title} =~ s/[-_]/ /gx;     # dashes and underscores to spaces
+   $option{title} =~ s/[-_]/ /xsmg;     # dashes and underscores to spaces
    $option{title} =~
-     s/\b(adf|cct|jpeg)\b/\U$1/gx;    # upper case comment abbreviations
-   $option{title} =~ s/(^\w)/\U$1/xg; # capitalise at the beginning of the line
+     s/\b(adf|cct|jpeg)\b/\U$1/xsmg;    # upper case comment abbreviations
+   $option{title} =~
+     s/(^\w)/\U$1/xsmg;    # capitalise at the beginning of the line
    given ( $option{title} ) {
     when ('L') {
      $option{title} = 'Top-left x';
@@ -273,16 +327,16 @@ sub options2hash {
    }
 
    # Parse option description based on an 8-character indent.
-   my $desc = '';
+   my $desc = $EMPTY;
    while (
     $output =~ /
-                       ^\ {8,}   # 8-character indent
-                       (.*)\n    # text
-                       ([\S\s]*) # rest of output
-                     /x
+                       \A\ {8,}   # 8-character indent
+                       ([^\n]*)\n    # text
+                       (.*) # rest of output
+                     /xsm
      )
    {
-    if ( $desc eq '' ) {
+    if ( $desc eq $EMPTY ) {
      $desc = $1;
     }
     else {
@@ -301,7 +355,7 @@ sub options2hash {
   push @options, \%option;
   $option{index} = $#options + 1;
  }
- unshift @options, { index => 0 } if (@options);
+ if (@options) { unshift @options, { index => 0 } }
  return \@options, \%hash;
 }
 
@@ -310,8 +364,9 @@ sub options2hash {
 sub parse_constraint {
  my ( $option, $values ) = @_;
  $option->{type} = SANE_TYPE_INT;
- $option->{type} = SANE_TYPE_FIXED
-   if ( defined( $option->{val} ) and $option->{val} =~ /\./x );
+ if ( defined( $option->{val} ) and $option->{val} =~ /\./xsm ) {
+  $option->{type} = SANE_TYPE_FIXED;
+ }
  if (
   defined($values)
   and $values =~ /
@@ -319,40 +374,44 @@ sub parse_constraint {
                     \.\.                   # two dots
                     (\d+\.?\d*)            # max value, possible floating
                     $units? # optional unit
-                    (,\.\.\.)? # multiple values
-                  /x
+                    ($list)? # multiple values
+                  /xsm
    )
  {
   $option->{constraint}{min} = $1;
   $option->{constraint}{max} = $2;
   $option->{constraint_type} = SANE_CONSTRAINT_RANGE;
-  $option->{unit}       = unit2enum($3) if ( defined $3 );
-  $option->{max_values} = 255           if ( defined $4 );
-  $option->{constraint}{quant} = $1
-    if (
+  if ( defined $3 ) { $option->{unit}       = unit2enum($3) }
+  if ( defined $4 ) { $option->{max_values} = $MAX_VALUES }
+  if (
    $values =~ /
                        \(              # opening round bracket
                        in\ steps\ of\  # text
                        (\d+\.?\d*)     # step
                        \)              # closing round bracket
-                     /x
-    );
-  $option->{type} = SANE_TYPE_FIXED
-    if (
-      $option->{constraint}{min} =~ /\./x
-   or $option->{constraint}{max} =~ /\./x
+                     /xsm
+    )
+  {
+   $option->{constraint}{quant} = $1;
+  }
+  if (
+      $option->{constraint}{min} =~ /\./xsm
+   or $option->{constraint}{max} =~ /\./xsm
    or ( defined( $option->{constraint}{quant} )
-    and $option->{constraint}{quant} =~ /\./x )
-    );
+    and $option->{constraint}{quant} =~ /\./xsm )
+    )
+  {
+   $option->{type} = SANE_TYPE_FIXED;
+  }
  }
- elsif ( defined($values) and $values =~ /^<(\w+)>(,\.\.\.)?$/x ) {
+ elsif ( defined($values) and $values =~ /^<(\w+)>($list)?$/xsm ) {
   if ( $1 eq 'float' ) {
    $option->{type} = SANE_TYPE_FIXED;
   }
   elsif ( $1 eq 'string' ) {
    $option->{type} = SANE_TYPE_STRING;
   }
-  $option->{max_values} = 255 if ( defined $2 );
+  if ( defined $2 ) { $option->{max_values} = $MAX_VALUES }
  }
 
  # if we haven't got a boolean, and there is no constraint, we have a button
@@ -368,29 +427,15 @@ sub parse_constraint {
 
 sub parse_list_constraint {
  my ( $option, $values ) = @_;
- if ( $values =~ /,\.\.\./x ) {
-  $option->{max_values} = 255;
-  $values = substr( $values, 0, length($values) - 4 );
+ if ( $values =~ /(.*)$list/xsm ) {
+  $values = $1;
+  $option->{max_values} = $MAX_VALUES;
  }
- my @array;
- while ( defined $values ) {
-  my $i = index( $values, '|' );
-  my $value;
-  if ( $i > -1 ) {
-   $value  = substr( $values, 0,      $i );
-   $values = substr( $values, $i + 1, length($values) );
-  }
-  else {
-   if ( $values =~ /$units$/x ) {
-    my $unit = $1;
-    $option->{unit} = unit2enum($unit);
-    $values = substr( $values, 0, index( $values, $unit ) );
-   }
-   $value = $values;
-   undef $values;
-  }
-  push @array, $value if ( $value ne '' );
+ if ( $values =~ /(.*)$units$/xsm ) {
+  $values = $1;
+  $option->{unit} = unit2enum($2);
  }
+ my @array = split( /\|+/sm, $values );
  if (@array) {
   if ( $array[0] eq 'auto' ) {
    $option->{cap} += SANE_CAP_AUTOMATIC;
@@ -411,10 +456,10 @@ sub parse_list_constraint {
 
    # Can't check before because 'auto' would mess things up
    for (@array) {
-    if (/[[:alpha:]]/x) {
+    if (/[[:alpha:]]/xsm) {
      $option->{type} = SANE_TYPE_STRING;
     }
-    elsif (/\./x) {
+    elsif (/\./xsm) {
      $option->{type} = SANE_TYPE_FIXED;
     }
    }
@@ -443,7 +488,7 @@ sub unit2enum {
   when ('dpi') {
    return SANE_UNIT_DPI;
   }
-  when ('%') {
+  when (q{%}) {
    return SANE_UNIT_PERCENT;
   }
   when ('us') {
