@@ -6,8 +6,13 @@ use warnings;
 use Carp;
 use File::Temp;    # To create temporary files
 use File::Basename;
-use Gscan2pdf::Document;    # for slurp
+use Gscan2pdf::Document;             # for slurp
 use version;
+use English qw( -no_match_vars );    # for $PROCESS_ID
+
+our $VERSION = '1.2.3';
+my $EMPTY = q{};
+my $COMMA = q{,};
 
 my ( %languages, $installed, $setup, $version, $tessdata, $datasuffix,
  $logger );
@@ -17,12 +22,12 @@ sub setup {
  return $installed if $setup;
 
  my ( $exe, undef ) = Gscan2pdf::Document::open_three('which tesseract');
- return if ( not defined($exe) or $exe eq '' );
+ return if ( not defined($exe) or $exe eq $EMPTY );
  $installed = 1;
 
 # if we have 3.02.01 or better, we can use --list-langs and not bother with tessdata
  my ( $out, $err ) = Gscan2pdf::Document::open_three("tesseract -v");
- if ( $err =~ /^tesseract\ ([\d\.]+)/x ) {
+ if ( $err =~ /^tesseract\ ([\d\.]+)/xsm ) {
   $version = $1;
  }
  if ( $version and version->parse("v$version") > version->parse('v3.02') ) {
@@ -34,10 +39,10 @@ sub setup {
  ( $out, $err ) = Gscan2pdf::Document::open_three("tesseract '' '' -l ''");
  ( $tessdata, $version, $datasuffix ) = parse_tessdata( $out . $err );
 
- unless ( defined $tessdata ) {
+ if ( not defined($tessdata) ) {
   if ( $version and version->parse("v$version") > version->parse('v3.01') ) {
    my ( $lib, undef ) = Gscan2pdf::Document::open_three("ldd $exe");
-   if ( $lib =~ /libtesseract\.so.\d+\ =>\ ([\/a-zA-Z0-9\-\.\_]+)\ /x ) {
+   if ( $lib =~ /libtesseract\.so.\d+\ =>\ ([\/a-zA-Z0-9\-\.\_]+)\ /xsm ) {
     ( $out, undef ) = Gscan2pdf::Document::open_three("strings $1");
     $tessdata = parse_strings($out);
    }
@@ -57,19 +62,19 @@ sub setup {
 
 sub parse_tessdata {
  my @output = @_;
- my $output = join ",", @output;
+ my $output = join $COMMA, @output;
  my ( $v, $suffix );
- if ( $output =~ /\ v(\d\.\d\d)\ /x ) {
+ if ( $output =~ /\ v(\d\.\d\d)\ /xsm ) {
   $v = $1 + 0;
  }
- if ( $output =~ /Unable\ to\ load\ unicharset\ file\ (.+)/x ) {
+ if ( $output =~ /Unable\ to\ load\ unicharset\ file\ ([^\n]+)/xsm ) {
   $output = $1;
-  $v      = 2 unless defined $v;
+  if ( not defined($v) ) { $v = 2 }
   $suffix = '.unicharset';
  }
- elsif ( $output =~ /Error\ openn?ing\ data\ file\ (.+)/x ) {
+ elsif ( $output =~ /Error\ openn?ing\ data\ file\ ([^\n]+)/xsm ) {
   $output = $1;
-  $v      = 3 unless defined $v;
+  if ( not defined($v) ) { $v = 3 }    ## no critic (ProhibitMagicNumbers)
   $suffix = '.traineddata';
  }
  elsif ( defined($v) and version->parse("v$v") > version->parse('v3.01') ) {
@@ -78,21 +83,21 @@ sub parse_tessdata {
  else {
   return;
  }
- $output =~ s/\/ $suffix $//x;
+ $output =~ s/\/ $suffix $//xsm;
  return $output, $v, $suffix;
 }
 
 sub parse_strings {
  my ($strings) = @_;
- my @strings = split "\n", $strings;
+ my @strings = split /\n/xsm, $strings;
  for (@strings) {
-  return $_ . "tessdata" if (/\/ share \//x);
+  return $_ . "tessdata" if (/\/ share \//xsm);
  }
  return;
 }
 
 sub languages {
- unless (%languages) {
+ if ( not %languages ) {
   my %iso639 = (
    ara        => 'Arabic',
    bul        => 'Bulgarian',
@@ -144,15 +149,15 @@ sub languages {
   if ( version->parse("v$version") > version->parse('v3.02') ) {
    my ( undef, $codes ) =
      Gscan2pdf::Document::open_three("tesseract --list-langs");
-   @codes = split "\n", $codes;
-   shift @codes if ( $codes[0] =~ /^List\ of\ available\ languages/x );
+   @codes = split /\n/xsm, $codes;
+   if ( $codes[0] =~ /^List\ of\ available\ languages/xsm ) { shift @codes }
   }
   else {
    for ( glob "$tessdata/*$datasuffix" ) {
 
     # Weed out the empty language files
     if ( not -z $_ ) {
-     if (/ ([\w\-]*) $datasuffix $/x) {
+     if (/ ([\w\-]*) $datasuffix $/xsm) {
       push @codes, $1;
      }
     }
@@ -177,7 +182,7 @@ sub hocr {
  # can't use the package-wide logger variable as we are in a thread here.
  ( my $class, my $file, my $language, $logger, my $pidfile ) = @_;
  my ( $tif, $cmd, $name, $path );
- Gscan2pdf::Tesseract->setup($logger) unless $setup;
+ if ( not $setup ) { Gscan2pdf::Tesseract->setup($logger) }
 
  # Temporary filename for output
  my $suffix =
@@ -186,7 +191,7 @@ sub hocr {
  ( $name, $path, undef ) = fileparse( $txt, $suffix );
 
  if ( version->parse("v$version") < version->parse("v3")
-  and $file !~ /\.tif$/x )
+  and $file !~ /\.tif$/xsm )
  {
 
   # Temporary filename for new file
@@ -211,14 +216,14 @@ sub hocr {
  $logger->info($cmd);
 
  # File in which to store the process ID so that it can be killed if necessary
- $cmd = "echo $$ > $pidfile;$cmd" if ( defined $pidfile );
+ if ( defined $pidfile ) { $cmd = "echo $PROCESS_ID > $pidfile;$cmd" }
 
  my ( $out, $err ) = Gscan2pdf::Document::open_three($cmd);
  my $warnings = $out . $err;
  my $leading  = 'Tesseract Open Source OCR Engine';
  my $trailing = 'with Leptonica';
- $warnings =~ s/$leading v\d\.\d\d $trailing\n//x;
- $warnings =~ s/^Page\ 0\n//x;
+ $warnings =~ s/$leading v\d\.\d\d $trailing\n//xsm;
+ $warnings =~ s/^Page\ 0\n//xsm;
  $logger->debug( 'Warnings from Tesseract: ', $warnings );
 
  return Gscan2pdf::Document::slurp($txt), $warnings;
